<?php
/*
 * ----------------------------------------------------------------------------
 * Va-Database.inc.php                                               10/18/2021
 * This script contains a set of functions that may be useful when dealing with
 * Virtual Advanced database groups, databases, and messages.
 * ----------------------------------------------------------------------------
 * Copyright (C) 2002-2021 Steve Winn. All Rights Reserved.
 *
 * For licensing information, view LICENSE.TXT.
 * ----------------------------------------------------------------------------
 */

require_once('include_dir.php');
require_once($includeDir . 'vadv-php.inc.php');


class vadvDatabases {
    /**
    * Va-Database.inc.php :: DbAccess()
    *
    * This function will determine if the user has access clearance for the specified
    * database.
    *
    * @param array $dbInfo
    * @param string $userNo
    * @param mixed $userInfo
    * @return boolean
    */
    function DbAccess($dbInfo,
                      $userNo = 0,
                      $userInfo = '') {
        if (!is_array($dbInfo)) {
            trigger_error(VA_Error(1));
        }
        if (!is_int($userNo)) {
            $userNo = (integer) $userNo;
        }

        if ((!isset($dbInfo)) ||
            (!isset($dbInfo['minreadsl']))) {
            return FALSE;
        }

        $classUserfileDat = new datUserfile;

        if (!is_array($userInfo)) {
            if ($userNo == 0) {
                $user = $GLOBALS['USER'];
            } else {
                $user = $classUserfileDat->Get($userNo);
            }
        } else {
            $user = $userInfo;
        }

        // Set the default access value, false.
        $access = FALSE;

        // If the sysop, give access regardless.
        if ($user['securitylevel'] >= 255) {
            return TRUE;
        }

        // Check security level, age and the user access flags.
        if (($user['securitylevel'] >= $dbInfo['minreadsl']) &&
            ($user['age'] >= $dbInfo['agelimit']) &&
            (($dbInfo['accflags'] == '') ||
             (strpos($user['accflags'], $dbInfo['accflags']) !== FALSE))) {
            $access = TRUE;
        }

        // Return the access result.
        return $access;
    }


    /**
    * Va-Database.inc.php :: DeleteMessage()
    *
    * This function will permanently delete the given message. It will delete any
    * attached file as well. BE SURE TO RELOAD THE DATABASE INFO AFTER USING THIS
    * FUNCTION SO THE NEW INFORMATION IS UPDATED.
    *
    * @param array $dbInfo
    * @param integer $msgNo
    * @return mixed
    */
    function DeleteMessage(&$dbInfo,
                           $msgNo) {
        if (!is_array($dbInfo)) {
            trigger_error(VA_Error(1));
        }
        if (!is_int($msgNo)) {
            $msgNo = (integer) $msgNo;
        }

        // Check to see if the message number is within the proper range.
        if (($msgNo > $dbInfo['msgno']) ||
            ($msgNo < 1)) {
            return FALSE;
        }

        $file = $GLOBALS['VA_SYSPATHS']['db'] . '/' . $dbInfo['filename'];
        $datFile = $file . '.DAT';
        $binFile = $file . '.BIN';
        $qscFile = $file . '.QSC';

        // Read the file string.
        $fileString = ReadCfgFile($datFile);
        if ($fileString === FALSE) {
            return FALSE;
        }

        $msgInfo = $this->GetMessageInfo($dbInfo, $msgNo, $fileString);

        // If an attached file, then delete it. Non-reversible!
        if ($msgInfo['attfilename'] != '') {
            if ($GLOBALS['CONFIG']['hostname'] != '') {
                $file = ConvertDir($GLOBALS['CONFIG']['hostname'], $dbInfo['filepath']) . '/' . $msgInfo['attfilename'];
            } else {
                $file = $dbInfo['filepath'] . '/' . $msgInfo['attfilename'];
            }

            if ($msgInfo['attfilepath'] == '') {
                if (file_exists($file)) {
                    unlink($file);
                }
            } else {
                if ($GLOBALS['CONFIG']['hostname'] != '') {
                    $userDir = ConvertUnc($GLOBALS['VA_SYSPATHS']['sysop'] . '/' . $GLOBALS['USER']['userno'] . '/' . $msgInfo['attfilename']);
                } else {
                    $userDir = $GLOBALS['VA_SYSPATHS']['sysop'] . '/' . $GLOBALS['USER']['userno'] . '/' . $msgInfo['attfilename'];
                }
                if ($msgInfo['attfilepath'] == $userDir) {
                    if ($GLOBALS['CONFIG']['hostname'] != '') {
                        $file = ConvertDir($GLOBALS['CONFIG']['hostname'], $msgInfo['attfilepath']);
                    } else {
                        $file = $msgInfo['attfilepath'];
                    }
                }
                if (file_exists($file)) {
                    unlink($file);
                }
            }
            $msgInfo['attfilesize'] = 0;
            $msgInfo['attfilename'] = '';
            $msgInfo['attfilepath'] = '';
        }

        // Remove the message from the .BIN file.
        $binFileString = ReadCfgFile($binFile);
        if ($binFileString !== FALSE) {
            $binFileString = substr_replace($binFileString, '', $msgInfo['msgpos'] - 1, $msgInfo['msglength']);
            $status = WriteCfgFile($binFile, $binFileString);
        }

        // Update all messages with new position information in the .DAT file.
        for ($i = $msgNo + 1; $i <= $dbInfo['msgno']; $i++) {
            $info = $this->GetMessageInfo($dbInfo, $i, $fileString);
            $info['msgpos'] = $info['msgpos'] - $msgInfo['msglength'];
            $fileString = substr_replace($fileString, MakeLongInt($info['msgpos']), (($i - 1) * 582) + 479 - 1, 4);
        }

        // Remove the message information from the .DAT file.
        $fileString = substr_replace($fileString, '', ($msgNo - 1) * 582, 582);
        $status = WriteCfgFile($datFile, $fileString);

        $qscFileString = ReadCfgFile($qscFile);
        if ($qscFileString !== FALSE) {
            $qscLength = intval(strlen($qscFileString) / 4);
            $offset = 0;
            // Update all user's pointers if they are now greater than the number of messages.
            for ($i = 1; $i <= $qscLength; $i++) {
                $pointer = GetLongInt($qscFileString, 1 + $offset);
                if (($pointer > $dbInfo['msgno'] - 1) &&
                    ($pointer < 32760)) {
                    $pointer = $dbInfo['msgno'] - 1;
                    $qscFileString = substr_replace($qscFileString, MakeLongInt($pointer), $offset, 4);
                }
                $offset = $offset + 4;
            }
            WriteCfgFile($qscFile, $qscFileString);
        }

        // Return the status.
        return $status;
    }


    /**
    * Va-Database.inc.php :: EditMessage()
    *
    * This function allows you to change the selected message's subject and/or
    * body. It will write the new message exactly as it is passed, so routing
    * information, color codes, etc. would need to be part of the new message. At
    * the end of the new message, it will write a diamond code with date/time
    * information of the change.
    *
    * @param array $dbInfo
    * @param integer $msgNo
    * @param string $newSubject
    * @param string $newBody
    * @return mixed
    */
    function EditMessage($dbInfo,
                         $msgNo,
                         $newSubject = '',
                         $newBody = '') {
        if (!is_array($dbInfo)) {
            trigger_error(VA_Error(1));
        }
        if (!is_int($msgNo)) {
            $msgNo = (integer) $msgNo;
        }
        if (!is_string($newSubject)) {
            $newSubject = (string)  $newSubject;
        }
        if (!is_string($newBody)) {
            $newBody = (string)  $newBody;
        }

        // Check to see if the message number is within the proper range.
        if (($msgNo > $dbInfo['msgno']) ||
            ($msgNo < 1)) {
            return FALSE;
        }

        $file = $GLOBALS['VA_SYSPATHS']['db'] . '/' . $dbInfo['filename'];
        $datFile = $file . '.DAT';
        $binFile = $file . '.BIN';

        // Read the file string.
        $fileString = ReadCfgFile($datFile);
        if ($fileString === FALSE) {
            return FALSE;
        }

        $msgInfo = $this->GetMessageInfo($dbInfo, $msgNo, $fileString);

        if ($newSubject != '') {
            // Update the subject of the message.
            $msgSubject = rtrim(strip_tags($newSubject));
            if (strlen($msgSubject) > 64) {
                $msgSubject = substr($msgSubject, 0, 64);
            } else {
                $msgSubject = str_pad($msgSubject, 64);
            }
            $fileString = substr_replace($fileString, $msgSubject, (($msgNo - 1) * 582) + 269 - 1, 64);
        }

        if ($newBody != '') {
            // Get the message from the .BIN file.
            // Open the file.
            $fp = fopen($binFile, 'r');
            fseek($fp, $msgInfo['msgpos'] - 1);

            // If the message length is greater than zero...
            if ($msgInfo['msglength'] > 0) {
                // ...get the message, otherwise...
                $message = fread($fp, $msgInfo['msglength']);
            } else {
                // ...the message is nothing.
                $message = '';
            }
             // Close the file.
            fclose($fp);

            // Update the body of the message.
            $msgBody = rtrim(wordwrap($newBody, 79, "\r\n", 1));
            $msgBody = $msgBody . "\r\n" . chr(4) . "0\r\n" . chr(4) . '0Last Edited: ' . date("D M d, Y H:i:s") . "\r\n";
            $msgLength = strlen($msgBody);
            $msgPosDiff = $msgInfo['msglength'] - $msgLength;

            $binFileString = ReadCfgFile($binFile);
            if ($binFileString !== FALSE) {
                $binFileString = substr_replace($binFileString, $msgBody, $msgInfo['msgpos'] - 1, $msgInfo['msglength']);
                $status = WriteCfgFile($binFile, $binFileString);
            }

            // Update the current message with new length information.
            $fileString = substr_replace($fileString, MakeLongInt($msgLength), (($msgNo - 1) * 582) + 483 - 1, 4);

            // Update all messages with new position information in the .DAT file.
            for ($i = $msgNo + 1; $i <= $dbInfo['msgno']; $i++) {
                $info = $this->GetMessageInfo($dbInfo, $i, $fileString);
                $info['msgpos'] = $info['msgpos'] - $msgPosDiff;
                $fileString = substr_replace($fileString, MakeLongInt($info['msgpos']), (($i - 1) * 582) + 479 - 1, 4);
            }
        }

        // Save the files.
        $status = WriteCfgFile($datFile, $fileString);

        // Return the status.
        return $status;
    }


    /**
    * Va-Database.inc.php :: GetDbCount()
    *
    * This function will get the number of available databases in the selected
    * topic. It will filter out those which the user does not have access to.
    *
    * @param integer $topicNo
    * @return integer
    */
    function GetDbCount($topicNo,
                        $filter = 1) {
        if (!is_int($topicNo)) {
            $topicNo = (integer) $topicNo;
        }
        if (!is_bool($filter)) {
            $filter = (boolean) $filter;
        }

        if ($topicNo < 0) {
            trigger_error(VA_Error(2));
        }

        $topic = chr($topicNo + 32);

        $file = $GLOBALS['VA_SYSPATHS']['main'] . '/DATABASE.CFG';
        if (!file_exists($file)) {
            return 0;
        }

        // Read the file.
        $fileString = ReadCfgFile($file);

        // Set the starting position.
        $position = 75;
        $dbNo = 0;

        for ($i = 0; $position < strlen($fileString); $i++) {
            $string = trim(strtoupper(substr($fileString, $position - 1, 1)));
            $position = $position + 131;
            if ($string == $topic) {
                $offset = 131 * $i;
                if ($filter) {
                    $dbInfo['minreadsl'] = GetInteger($fileString, 78 + $offset);
                    $dbInfo['agelimit'] = GetInteger($fileString, 82 + $offset);
                    $dbInfo['accflags'] = GetAccFlags(GetLongInt($fileString, 84 + $offset));
                    if ($this->DbAccess($dbInfo)) {
                        $dbNo++;
                    }
                } else {
                    $dbNo++;
                }
            }
        }

        // Return the database count.
        return $dbNo;
    }


    /**
    * Va-Database.inc.php :: GetDbInfo()
    *
    * This function will retrieve the variables for the selected database. You may
    * optionally pass the filestring, which would greatly reduce execution time if
    * you are to be searching the same database multiple times.
    *
    * @param integer $topicNo
    * @param integer $db
    * @param string $fileString
    * @return array
    */
    function GetDbInfo($topicNo,
                       $db,
                       $fileString = '') {
        if (!is_int($topicNo)) {
            $topicNo = (integer) $topicNo;
        }
        if (!is_int($db)) {
            $db = (integer) $db;
        }
        if (!is_string($fileString)) {
            $fileString = (string) $fileString;
        }

        $dbInfo = array();

        if ($topicNo < 1) {
            $topicNo = 1;
        }
        if ($db < 1) {
            $db = 1;
        }

        $topic = chr($topicNo + 32);

        if (trim($fileString == '')) {
            // Set the filename.
            $file = $GLOBALS['VA_SYSPATHS']['main'] . '/DATABASE.CFG';
            if (!file_exists($file)) {
                return $dbInfo;
            }

            // Read the file.
            $fileString = ReadCfgFile($file);
        }

        $dbInfo['topicno'] = '';
        $dbInfo['topic'] = '';
        $dbInfo['dbno'] = 0;
        $dbInfo['filename'] = '';
        $dbInfo['filepath'] = '';
        $dbInfo['desc'] = '';
        $dbInfo['private'] = 0;
        $dbInfo['dbgroup'] = '';
        $dbInfo['maxmsg'] = 0;
        $dbInfo['minreadsl'] = 0;
        $dbInfo['minwritesl'] = 0;
        $dbInfo['agelimit'] = 0;
        $dbInfo['accflags'] = 0;
        $dbInfo['tagline'] = 0;
        $dbInfo['dispname'] = 0;
        $dbInfo['randtitle'] = 0;
        $dbInfo['qwkconfno'] = 0;
        $dbInfo['moderator'] = 0;
        $dbInfo['validate'] = 0;
        $dbInfo['freqable'] = 0;
        $dbInfo['devicetype'] = 0;
        $dbInfo['dbtype'] = '';
        $dbInfo['extra'] = '';
        $dbInfo['msgno'] = 0;
        $dbInfo['msgcnt'] = 0;
        $dbInfo['networked'] = FALSE;

        $position = 75;
        $num = 0;

        for ($i = 0; $position < strlen($fileString); $i++) {
            $string = trim(strtoupper(substr($fileString, $position - 1, 1)));
            $position = $position + 131;
            if ($string == $topic) {
                $num++;
                if ($num == $db) {
                    $offset = 131 * $i;
                    $dbInfo['topicno'] = $topicNo;
                    $dbInfo['topic'] = $topic;
                    $dbInfo['dbno'] = $db;
                    $dbInfo['filename'] = GetString($fileString, 1 + $offset, 8);
                    $dbInfo['filepath'] = GetString($fileString, 9 + $offset, 32);
                    if (dirname($dbInfo['filepath']) == '.') {
                        $dbInfo['filepath'] = $GLOBALS['VA_SYSPATHS']['main'] . '/' . $dbInfo['filepath'];
                    }
                    $dbInfo['desc'] = GetString($fileString, 41 + $offset, 32);
                    $dbInfo['private'] = GetInteger($fileString, 73 + $offset);
                    $dbInfo['dbgroup'] = GetString($fileString, 75 + $offset, 1);
                    $dbInfo['maxmsg'] = GetInteger($fileString, 76 + $offset);
                    $dbInfo['minreadsl'] = GetInteger($fileString, 78 + $offset);
                    $dbInfo['minwritesl'] = GetInteger($fileString, 80 + $offset);
                    $dbInfo['agelimit'] = GetInteger($fileString, 82 + $offset);
                    $dbInfo['accflags'] = GetAccFlags(GetLongInt($fileString, 84 + $offset));
                    $dbInfo['tagline'] = GetInteger($fileString, 88 + $offset);
                    $dbInfo['dispname'] = GetInteger($fileString, 90 + $offset);
                    $dbInfo['randtitle'] = GetInteger($fileString, 92 + $offset);
                    $dbInfo['qwkconfno'] = GetInteger($fileString, 94 + $offset);
                    $dbInfo['moderator'] = GetInteger($fileString, 96 + $offset);
                    $dbInfo['validate'] = GetInteger($fileString, 98 + $offset);
                    $dbInfo['freqable'] = GetInteger($fileString, 100 + $offset);
                    $dbInfo['devicetype'] = GetInteger($fileString, 102 + $offset);
                    $dbInfo['dbtype'] = GetString($fileString, 104 + $offset, 1);
                    $dbInfo['extra'] = GetString($fileString, 105 + $offset, 27);
                    $dbInfo['msgno'] = $this->GetMessageCount($dbInfo);
                    $dbInfo['msgcnt'] = $dbInfo['msgno'];
                    break;
                }
            }
        }

        if ((isset($dbInfo['filename'])) &&
            (trim($dbInfo['filename']) != '')) {
            $file = $GLOBALS['VA_SYSPATHS']['db'] . '/' . $dbInfo['filename'] . '.NET';
            if ((file_exists($file)) &&
                (filesize($file) > 0)) {
                $dbInfo['networked'] = TRUE;
            } else {
                $dbInfo['networked'] = FALSE;
            }
        }

        // Return the database information.
        return $dbInfo;
    }


    /**
    * Va-Database.inc.php :: GetDbNetworks()
    *
    * This function will get any network information for the database passed to it
    * and return the results in an array.
    *
    * @param array $dbInfo
    * @return array
    */
    function GetDbNetworks($dbInfo) {
        if (!is_array($dbInfo)) {
            trigger_error(VA_Error(1));
        }

        $networks[0] = 0;

        $file = $GLOBALS['VA_SYSPATHS']['db'] . '/' . $dbInfo['filename'] . '.NET';
        if (!file_exists($file)) {
            return $networks;
        }
        $fileArray = file($file);

        for ($i = 0; $i < count($fileArray); $i++) {
            $info = explode('=', $fileArray[$i]);
            if ((trim($fileArray[$i]) != '') &&
                (count($info) !== FALSE)) {
                $networks[$i + 1]['network'] = $info[0];
                $networks[$i + 1]['id'] = $info[1];
            }
        }

        $networks[0] = $i;

        // Return the array results.
        return $networks;
    }


    /**
    * Va-Database.inc.php :: GetEmailCount()
    *
    * This function will determine the number of messages contained within the
    * email database for the specified user. It returns an array containing
    * the number of messages total (msgno) and the number of messages that have not
    * been read (unread) and the number of deleted messages (deleted).
    *
    * If deleted is set to 0 (false), then it will only return message numbers of
    * messages that are not marked for deletion.
    *
    * It also returns the email message numbers starting with $count[0].
    *
    * @param integer $userNo
    * @return array
    */
    function GetEmailCount($userNo,
                           $deleted = 1) {
        if (!is_int($userNo)) {
            $userNo = (integer) $userNo;
        }
        if (!is_bool($deleted)) {
            $deleted = (boolean) $deleted;
        }

        // Set the default value.
        $count['msgno'] = 0;
        $count['msgtotal'] = 0;
        $count['unread'] = 0;
        $count['deleted'] = 0;

        if ($userNo < 1) {
            return $count;
        }

        // Set the filename.
        $datFile = $GLOBALS['VA_SYSPATHS']['db'] . '/EMAIL.DAT';
        if (!file_exists($datFile)) {
            return $count;
        }

        // Read the file.
        $fileString = ReadCfgFile($datFile);

        $counter = 0;
        $offset = 0;
        for ($i = 0; $offset < filesize($datFile); $i++) {
            $offset = 582 * $i;
            $user = GetLongInt($fileString, 5 + $offset);
            if ($user == $userNo) {
                $delFlag = GetInteger($fileString, 333 + $offset);
                if ($delFlag == 1) {
                    $count['deleted']++;
                }
                $readFlag = GetLongInt($fileString, 375 + $offset);
                if ($readFlag == 0) {
                    $count['unread']++;
                }
                if ($deleted) {
                    $count[$counter] = $i + 1;
                    $counter++;
                    $count['msgno']++;
                } else {
                    if ($delFlag == 0) {
                        $count[$counter] = $i + 1;
                        $counter++;
                        $count['msgno']++;
                    }
                }
                $count['msgtotal']++;
            }
        }

        // Return the message count.
        return $count;
    }


    /**
    * Va-Database.inc.php :: GetLastPostInfo()
    *
    * This function will get the last post's message information of the
    * specified database.
    *
    * @param array $dbInfo
    * @return array
    */
    function GetLastPostInfo($dbInfo) {
        if (!is_array($dbInfo)) {
            trigger_error(VA_Error(1));
        }

        $lastPost = $this->GetMessageInfo($dbInfo, $dbInfo['msgno']);

        // Return the lastpost information.
        return $lastPost;
    }


    /**
    * Va-Database.inc.php :: GetMessage()
    *
    * This function will retrieve the message for the selected information. You may
    * set $format to 1 if you would like to strip html tags or 2 for no formatting.
    *
    * @param array $dbInfo
    * @param integer $msgNo
    * @param integer $format
    * @return string
    */
    function GetMessage($dbInfo,
                        $msgNo,
                        $format = 0,
                        $fileString = '',
                        $msgInfo = array()) {
        if (!is_array($dbInfo)) {
            trigger_error(VA_Error(1));
        }
        if (!is_int($msgNo)) {
            $msgNo = (integer) $msgNo;
        }
        if (!is_int($format)) {
            $format = (integer) $format;
        }
        if (!is_string($fileString)) {
            $fileString = (string) $fileString;
        }
        if (!is_array($msgInfo)) {
            trigger_error(VA_Error(1));
        }

        if ($msgNo < 1) {
            $msgNo = 1;
        }
        if ($msgNo > $dbInfo['msgno']) {
            return '';
        }
        if (intval($format) > 2) {
            $format = 1;
        }

        // Set the filename.
        $file = $GLOBALS['VA_SYSPATHS']['db'] . '/' . $dbInfo['filename'];
        $binFile = $file . '.BIN';
        if (!file_exists($binFile)) {
            return '';
        }

        if (count($msgInfo) == 0) {
            $msgInfo = $this->GetMessageInfo($dbInfo, $msgNo);
        }

        if (trim($fileString) == '') {
            // Open the file.
            $fp = fopen($binFile, 'r');
            fseek($fp, $msgInfo['msgpos'] - 1);

            // If the message length is greater than zero...
            if ($msgInfo['msglength'] > 0) {
                // get the message, otherwise...
                $message = fread($fp, $msgInfo['msglength']);
            } else {
                // the message is nothing.
                $message = '';
            }
            // Close the file.
            fclose($fp);
        } else {
            if ($msgInfo['msglength'] > 0) {
                $message = substr($fileString, $msgInfo['msgpos'] - 1, $msgInfo['msglength']);
            } else {
                $message = '';
            }
        }

        // HTML formatting...
        if ($format == 1) {
            $message = rtrim(preg_replace("/(=20)||(=09)/", '', strip_tags($message)));
        } else if ($format == 2) {
            $message = rtrim($message);
        } else if ($format == 3) {
            $message = rtrim(htmlentities(HTMLEncode($message, $GLOBALS['USER_SETTINGS']['htmlencode']), ENT_COMPAT, 'cp866'));
        } else {
            $message = rtrim(str_replace('&amp;', '&', htmlspecialchars($message)));
        }

        while ((substr($message, 0, 1) == "\n") || (substr($message, 0, 1) == "\r")) {
            $message = substr($message, 1);
        }

        // Detect MIME header and only display text version.
        $pos = strpos(ltrim($message), "\r\n");
        $line = substr(ltrim($message), 0, $pos);
        if (is_int(strpos($line, 'MIME'))) {
            $tempMessage = ltrim(substr(ltrim($message), $pos + 1));
        } else {
            $tempMessage = ltrim($message);
        }
        $line = substr($tempMessage, 0, 2);
        if ($line == '--') {
            $pos = strpos($tempMessage, "\r\n");
            $line = ltrim(substr($tempMessage, $pos + 1));
            if (strtolower(substr($line, 0, 8)) == 'content-') {
                $mimehdr = ltrim(substr($tempMessage, 0, $pos));
                $pos = strpos($tempMessage, "\r\n\r\n");
                $message = ltrim(substr($tempMessage, $pos + 1));
                $pos = strpos($message, $mimehdr);
                $message = trim(substr($message, 0, $pos));
            }
        }

        $array = explode("\r\n", $message);
        foreach ($array as $key => $value) {
            $array[$key] = rtrim($value);
        }
        $message = implode("\r\n", $array);

        // Return the message.
        return $message;
    }


    /**
    * Va-Database.inc.php :: GetMessageCount()
    *
    * This function will determine the number of messages contained within the
    * specified database. If $includeDeleted is set to FALSE, then any messages
    * marked as deleted will not be counted.
    *
    * @param array $dbInfo
    * @param boolean $includeDeleted
    * @return integer
    */
    function GetMessageCount($dbInfo,
                             $includeDeleted = 1) {
        if (!is_array($dbInfo)) {
            trigger_error(VA_Error(1));
        }
        if (!is_bool($includeDeleted)) {
            $includeDeleted = (boolean) $includeDeleted;
        }

        // Set the filename.
        $file = $GLOBALS['VA_SYSPATHS']['db'] . '/' . $dbInfo['filename'];

        // Add the datafile extension.
        $datFile = $file . '.DAT';
        if (!file_exists($datFile)) {
            return 0;
        }

        // If $includeDeleted is true, then just calculate the number of messages,
        // otherwise, view each message entry and check the deleted status.
        if ($includeDeleted) {
            // Determine the size of the file.
            $fileSize = filesize($datFile);

            // Determine the number of messages.
            $msgNo = intval($fileSize / 582);
        } else {
            // Read the database.
            $msgNo = 0;
            $fileString = ReadCfgFile($datFile);
            for ($i = 1; $i <= intval(filesize($datFile) / 582); $i++) {
                $message = $this->GetMessageInfo($dbInfo, $i, $fileString);
                if ($message['deleted'] == 0) {
                    $msgNo++;
                }
            }
        }

        // Return the message count.
        return $msgNo;
    }


    /**
    * Va-Database.inc.php :: GetMessageInfo()
    *
    * This function will retrieve the variables for the specified message. You may
    * pass the filestring as well, which would greatly reduce the execution time
    * if you do multiple searches.
    *
    * @param array $dbInfo
    * @param integer $msgNo
    * @param string $fileString
    * @return array
    */
    function GetMessageInfo($dbInfo,
                            $msgNo,
                            $fileString = '') {
        if (!is_array($dbInfo)) {
            trigger_error(VA_Error(1));
        }
        if (!is_int($msgNo)) {
            $msgNo = (integer) $msgNo;
        }
        if (!is_string($fileString)) {
            $fileString = (string) $fileString;
        }

        $msgInfo = array();
        $msgInfo['topicno'] = $dbInfo['topicno'];
        $msgInfo['topic'] = $dbInfo['topic'];
        $msgInfo['dbno'] = $dbInfo['dbno'];
        $msgInfo['fromuserno'] = 0;
        $msgInfo['touserno'] = 0;
        $msgInfo['fromhandle'] = '';
        $msgInfo['tohandle'] = '';
        $msgInfo['fromnetnode'] = '';
        $msgInfo['tonetnode'] = '';
        $msgInfo['fromnetid'] = 0;
        $msgInfo['tonetid'] = 0;
        $msgInfo['timestamp'] = 0;
        $msgInfo['daysold'] = 0;
        $msgInfo['sysmsgno'] = 0;
        $msgInfo['threadbk'] = 0;
        $msgInfo['threadbkn'] = 0;
        $msgInfo['subject'] = '';
        $msgInfo['deleted'] = 0;
        $msgInfo['creation'] = '';
        $msgInfo['date'] = '';
        $msgInfo['time'] = '';
        $msgInfo['timezone'] = '';
        $msgInfo['offline'] = 0;
        $msgInfo['attfilesize'] = 0;
        $msgInfo['attfilename'] = '';
        $msgInfo['attfilepath'] = '';
        $msgInfo['lib'] = 0;
        $msgInfo['downloadno'] = 0;
        $msgInfo['dbid'] = 0;
        $msgInfo['filename'] = '';
        $msgInfo['msgpos'] = 0;
        $msgInfo['msglength'] = 0;
        $msgInfo['originnet'] = 0;
        $msgInfo['fidoflags'] = 0;
        $msgInfo['originnode'] = 0;
        $msgInfo['reserved'] = '';
        $msgInfo['extra'] = '';

        if ($msgNo < 1) {
            $msgNo = 1;
        }
        if ($msgNo > $dbInfo['msgno']) {
            return $msgInfo;
        }

        global $includeDir;

        if (trim($fileString) == '') {
            // Set the filename.
            $file = $GLOBALS['VA_SYSPATHS']['db'] . '/' . $dbInfo['filename'];

            // Set the database filename.
            $datFile = $file . '.DAT';
            if (!file_exists($datFile)) {
                return $msgInfo;
            }

            // Read the file.
            $fileString = ReadCfgFile($datFile);
        }

        $offset = 582 * ($msgNo - 1);
        $msgInfo['topicno'] = $dbInfo['topicno'];
        $msgInfo['topic'] = $dbInfo['topic'];
        $msgInfo['dbno'] = $dbInfo['dbno'];
        $msgInfo['fromuserno'] = GetLongInt($fileString, 1 + $offset);
        $msgInfo['touserno'] = GetLongInt($fileString, 5 + $offset);
        $msgInfo['fromhandle'] = GetString($fileString, 9 + $offset, 40);
        $msgInfo['tohandle'] = GetString($fileString, 49 + $offset, 40);
        $msgInfo['fromnetnode'] = GetString($fileString, 89 + $offset, 80);
        $msgInfo['tonetnode'] = GetString($fileString, 169 + $offset, 80);
        $msgInfo['fromnetid'] = GetInteger($fileString, 249 + $offset);
        $msgInfo['tonetid'] = GetInteger($fileString, 251 + $offset);
        $msgInfo['timestamp'] = GetLongInt($fileString, 253 + $offset);
        $msgInfo['daysold'] = intval((lTime() - $msgInfo['timestamp']) / (60 * 60 * 24));
        $msgInfo['sysmsgno'] = GetLongInt($fileString, 257 + $offset);
        $msgInfo['threadbk'] = GetLongInt($fileString, 261 + $offset);
        $msgInfo['threadbkn'] = GetLongInt($fileString, 265 + $offset);
        $msgInfo['subject'] = GetString($fileString, 269 + $offset, 64);
        $msgInfo['deleted'] = GetInteger($fileString, 333 + $offset);
        $msgInfo['creation'] = GetString($fileString, 335 + $offset, 40);
        $msgInfo['date'] = GetString($fileString, 335 + $offset, 17);
        $msgInfo['time'] = GetString($fileString, 352 + $offset, 9);
        $msgInfo['timezone'] = GetString($fileString, 361 + $offset, 14);
        $msgInfo['offline'] = GetLongInt($fileString, 375 + $offset);
        $msgInfo['attfilesize'] = GetLongInt($fileString, 379 + $offset);
        $msgInfo['attfilename'] = GetString($fileString, 383 + $offset, 12);
        $msgInfo['attfilepath'] = GetString($fileString, 395 + $offset, 64);
        $msgInfo['lib'] = GetLongInt($fileString, 459 + $offset);
        $msgInfo['downloadno'] = GetLongInt($fileString, 463 + $offset);
        $msgInfo['dbid'] = GetLongInt($fileString, 467 + $offset);
        $msgInfo['filename'] = GetString($fileString, 471 + $offset, 8);
        $msgInfo['msgpos'] = GetLongInt($fileString, 479 + $offset);
        $msgInfo['msglength'] = GetLongInt($fileString, 483 + $offset);
        $msgInfo['originnet'] = GetInteger($fileString, 487 + $offset);
        $msgInfo['fidoflags'] = GetInteger($fileString, 489 + $offset);
        $msgInfo['originnode'] = GetLongInt($fileString, 491 + $offset);
        $msgInfo['reserved'] = GetString($fileString, 495 + $offset, 24);
        $msgInfo['extra'] = GetString($fileString, 519 + $offset, 64);

        // Fix any missing information...
        if ($msgInfo['fromhandle'] == '') {
            if ($msgInfo['fromuserno'] == 0) {
                $msgInfo['fromhandle'] = 'Unknown';
            } else {
                $msgInfo['fromhandle'] = $msgInfo['fromuserno'];
            }
        }
        if ($msgInfo['tohandle'] == '') {
            if ($msgInfo['touserno'] == 0) {
                $msgInfo['tohandle'] = 'All';
            } else {
                $msgInfo['tohandle'] = $msgInfo['touserno'];
            }
        }

        // Return the message information.
        return $msgInfo;
    }


    /**
    * Va-Database.inc.php :: GetMessagePointer()
    *
    * This function will return the specified user's message pointer for the
    * current database.
    *
    * @param array $dbInfo
    * @param integer $userNo
    * @return integer
    */
    function GetMessagePointer($dbInfo,
                               $userNo,
                               $fileString = '') {
        if (!is_array($dbInfo)) {
            trigger_error(VA_Error(1));
        }
        if (!is_int($userNo)) {
            $userNo = (integer) $userNo;
        }
        if (!is_string($fileString)) {
            $fileString = (string) $fileString;
        }

        if ($userNo < 1) {
            return 32760;
        }

        $file = $GLOBALS['VA_SYSPATHS']['db'] . '/' . $dbInfo['filename'] . '.QSC';
        if ($fileString == '') {
            if (!file_exists($file)) {
                return 0;
            }
            $fileString = ReadCfgFile($file);
        }

        $offset = ($userNo - 1) * 4;
        $pointer = GetLongInt($fileString, 1 + $offset);

        // Return the message pointer.
        return $pointer;
    }


    /**
    * Va-Database.inc.php :: GetOnelinerCount()
    *
    * This function will determine the number of messages contained within the
    * onelner database for the specified user. It returns an array containing
    * the number of messages total (msgno) and the number of messages that have not
    * been read (unread) and the number of deleted messages (deleted).
    *
    * @param integer $userNo
    * @return integer
    */
    function GetOnelinerCount($userNo) {
        if (!is_int($userNo)) {
            $userNo = (integer) $userNo;
        }

        // Set the default value.
        $count['msgno'] = 0;
        $count['unread'] = 0;
        $count['deleted'] = 0;

        if ($userNo < 1) {
            return $count;
        }

        // Set the filename.
        $datFile = $GLOBALS['VA_SYSPATHS']['db'] . '/ONELINE.DAT';
        if (!file_exists($datFile)) {
            return $count;
        }

        // Read the file.
        $fileString = ReadCfgFile($datFile);

        $offset = 582;
        for ($i = 0; $offset < filesize($datFile); $i++) {
            $offset = 582 * $i;
            $user = GetLongInt($fileString, 5 + $offset);
            if ($user == $userNo) {
                $delFlag = GetInteger($fileString, 333 + $offset);
                if ($delFlag == 1) {
                    $count['deleted']++;
                }
                $readFlag = GetLongInt($fileString, 375 + $offset);
                if ($readFlag == 0) {
                    $count['unread']++;
                }
                $count['msgno']++;
            }
        }

        // Return the message count.
        return $count;
    }


    /**
    * Va-Database.inc.php :: GetSentEmailCount()
    *
    * This function will determine the number of messages contained within the
    * sent email database for the specified user. It returns an array containing
    * the number of messages total (msgno) and the number of messages that have not
    * been read (unread) and the number of deleted messages (deleted).
    *
    * If deleted is set to 0 (false), then it will only return message numbers of
    * messages that are not marked for deletion.
    *
    * It also returns the email message numbers starting with $count[0].
    *
    * @param integer $userNo
    * @return array
    */
    function GetSentEmailCount($userNo,
                               $deleted = 1) {
        if (!is_int($userNo)) {
            $userNo = (integer) $userNo;
        }
        if (!is_bool($deleted)) {
            $deleted = (boolean) $deleted;
        }

        // Set the default value.
        $count['msgno'] = 0;
        $count['msgtotal'] = 0;
        $count['unread'] = 0;
        $count['deleted'] = 0;

        if ($userNo < 1) {
            return $count;
        }

        // Set the filename.
        $datFile = $GLOBALS['VA_SYSPATHS']['db'] . '/SNTEMAIL.DAT';
        if (!file_exists($datFile)) {
            return $count;
        }

        // Read the file.
        $fileString = ReadCfgFile($datFile);

        $counter = 0;
        $offset = 0;
        for ($i = 0; $offset < filesize($datFile); $i++) {
            $offset = 582 * $i;
            $user = GetLongInt($fileString, 1 + $offset);
            if ($user == $userNo) {
                $delFlag = GetInteger($fileString, 333 + $offset);
                if ($delFlag == 1) {
                    $count['deleted']++;
                }
                $readFlag = GetLongInt($fileString, 375 + $offset);
                if ($readFlag == 0) {
                    $count['unread']++;
                }
                if ($deleted) {
                    $count[$counter] = $i + 1;
                    $counter++;
                    $count['msgno']++;
                } else {
                    if ($delFlag == 0) {
                        $count[$counter] = $i + 1;
                        $counter++;
                        $count['msgno']++;
                    }
                }
                $count['msgtotal']++;
            }
        }

        // Return the message count.
        return $count;
    }


    /**
    * Va-Database.inc.php :: GetTopicCount()
    *
    * This function will determine the total number of topics and return that value.
    *
    * @param string $type
    * @return integer
    */
    function GetTopicCount($type = '') {
        if (!is_string($type)) {
            $type = (string) $type;
        }

        $topicNo = 0;

        // Set the filename.
        $file = $GLOBALS['VA_SYSPATHS']['main'] . '/DBGROUP.CFG';
        if (!file_exists($file)) {
            return $topicNo;
        }

        // Read the file.
        $fileString = ReadCfgFile($file);

        // Change the topic type into uppercase.
        $type = trim(strtoupper($type));

        // Set the starting position.
        $position = 2;

        for ($i = 0; $position < strlen($fileString); $i++) {
            $string = trim(strtoupper(substr($fileString, $position - 2, 1)));
            if ((($type == '') ||
                (($type != '') &&
                 (strpos($type, $string) !== FALSE))) &&
                 ($string != '')) {
                $group = trim(strtoupper(substr($fileString, $position - 1, 1)));
                $group = ord($group) - 32;
                if ($this->GetDbCount($group) > 0) {
                    $topicNo++;
                }
            }
            $position = $position + 128;
        }

        // Return the topic count.
        return $topicNo;
    }


    /**
    * Va-Database.inc.php :: GetTopicInfo()
    *
    * This function will retrieve the variables for the selected database topic.
    * You can pass the db type to restrict which databases are searched. You may
    * also pass the filestring, which will greatly reduce execution time if you
    * do multiple searches.
    *
    * @param integer $topicNo
    * @param string $type
    * @param string $fileString
    * @return array
    */
    function GetTopicInfo($topicNo,
                          $type = '',
                          $fileString = '') {
        if (!is_int($topicNo)) {
            $topicNo = (integer) $topicNo;
        }
        if (!is_string($type)) {
            $type = (string) $type;
        }
        if (!is_string($fileString)) {
            $fileString = (string) $fileString;
        }

        $topicInfo = array();
        $topicInfo['topicno'] = 0;
        $topicInfo['topic'] = '';
        $topicInfo['dbtype'] = '';
        $topicInfo['topic'] = '';
        $topicInfo['desc'] = '';
        $topicInfo['futuretopic'] = '';
        $topicInfo['marker1'] = 0;
        $topicInfo['marker2'] = 0;
        $topicInfo['extra'] = '';
        $topicInfo['dbno'] = 0;

        if ($topicNo < 1) {
            $topicNo = 1;
        }

        if (trim($fileString) == '') {
            // Set the filename.
            $file = $GLOBALS['VA_SYSPATHS']['main'] . '/DBGROUP.CFG';
            if (!file_exists($file)) {
                return $topicInfo;
            }

            // Read the file.
            $fileString = ReadCfgFile($file);
        }

        // Change the type to uppercase.
        $type = trim(strtoupper($type));
        $topic = chr($topicNo + 32);

        // Set the starting position.
        $position = 2;

        for ($i = 0; $position < strlen($fileString); $i++) {
            $dbType = substr($fileString, $position - 2, 1);
            if (($type == '') ||
                (($type != '') &&
                 (strpos($type, $dbType) !== FALSE))) {
                $string = trim(strtoupper(substr($fileString, $position - 1, 1)));
                if ($string == $topic) {
                    $offset = 128 * $i;
                    $topicInfo['topicno'] = $topicNo;
                    $topicInfo['topic'] = $topic;
                    $topicInfo['dbtype'] = GetString($fileString, 1 + $offset, 1);
                    $topicInfo['topic'] = GetString($fileString, 2 + $offset, 1);
                    $topicInfo['desc'] = GetString($fileString, 3 + $offset, 65);
                    $topicInfo['futuretopic'] = GetString($fileString, 68 + $offset, 2);
                    $topicInfo['marker1'] = GetLongInt($fileString, 70 + $offset);
                    $topicInfo['marker2'] = GetLongInt($fileString, 74 + $offset);
                    $topicInfo['extra'] = GetString($fileString, 78 + $offset, 51);
                    $topicInfo['dbno'] = $this->GetDBCount($topicNo);
                    break;
                }
            }
            $position = $position + 128;
        }

        // Return the topic information.
        return $topicInfo;
    }


    /**
    * Va-Database.inc.php :: GetTopics()
    *
    * This function will retrieve the topic numbers and store them into an array.
    *
    * @param string $type
    * @return array
    */
    function GetTopics($type = '') {
        if (!is_string($type)) {
            $type = (string) $type;
        }

        $topics[0] = 0;

        // Set the filename.
        $file = $GLOBALS['VA_SYSPATHS']['main'] . '/DBGROUP.CFG';
        if (!file_exists($file)) {
            return $topics;
        }

        // Read the file.
        $fileString = ReadCfgFile($file);

        // Change the topic type into uppercase.
        $type = trim(strtoupper($type));

        // Set the starting position.
        $position = 2;

        for ($i = 0; $position < strlen($fileString); $i++) {
            $string = trim(strtoupper(substr($fileString, $position - 2, 1)));
            if ((($type == '') ||
                (($type != '') &&
                 (strpos($type, $string) !== FALSE))) &&
                 ($string != '')) {
                $group = trim(strtoupper(substr($fileString, $position - 1, 1)));
                $group = ord($group) - 32;
                if ($this->GetDbCount($group) > 0) {
                    array_push($topics, $group);
                }
            }
            $position = $position + 128;
        }

        $topics[0] = count($topics) - 1;

        // Return the topic numbers.
        return $topics;
    }


    /**
    * Va-Database.inc.php :: GetTotalMessageCount()
    *
    * This function will return the total number of database entries.
    *
    * @return integer
    */
    function GetTotalMessageCount() {
        $file = $GLOBALS['VA_SYSPATHS']['data'] . '/DBSEQ.DAT';
        if (!file_exists($file)) {
            return 0;
        }

        // Read the file.
        $fileString = ReadCfgFile($file);

        // Return the total message count.
        return GetLongInt($fileString, 1);
    }


    /**
    * Va-Database.inc.php :: PutDbInfo()
    *
    * This function will take the database information that is passed and create
    * a new entry into the database.cfg file.
    *
    * @param array $dbInfo
    * @return boolean
    */
    function PutDbInfo($dbInfo) {
        if (!is_array($dbInfo)) {
            trigger_error(VA_Error(1));
        }

        // Set the filename.
        $file = $GLOBALS['VA_SYSPATHS']['main'] . '/DATABASE.CFG';
        if (!file_exists($file)) {
            return FALSE;
        }

        // Create the new blank string.
        $newString = str_pad('', 131, chr(0));

        $newString = PutString($newString, $dbInfo['filename'], 1, 8);
        $newString = PutString($newString, $dbInfo['filepath'], 9, 32);
        $newString = PutString($newString, $dbInfo['desc'], 41, 32);
        $newString = PutInteger($newString, $dbInfo['private'], 73);
        $newString = PutString($newString, $dbInfo['dbgroup'], 75, 1);
        $newString = PutInteger($newString, $dbInfo['maxmsg'], 76);
        $newString = PutInteger($newString, $dbInfo['minreadsl'], 78);
        $newString = PutInteger($newString, $dbInfo['minwritesl'], 80);
        $newString = PutInteger($newString, $dbInfo['agelimit'], 82);
        $newString = PutLongInt($newString, MakeAccFlags($dbInfo['accflags']), 84);
        $newString = PutInteger($newString, $dbInfo['tagline'], 88);
        $newString = PutInteger($newString, $dbInfo['dispname'], 90);
        $newString = PutInteger($newString, $dbInfo['randtitle'], 92);
        $newString = PutInteger($newString, $dbInfo['qwkconfno'], 94);
        $newString = PutInteger($newString, $dbInfo['moderator'], 96);
        $newString = PutInteger($newString, $dbInfo['validate'], 98);
        $newString = PutInteger($newString, $dbInfo['freqable'], 100);
        $newString = PutInteger($newString, $dbInfo['devicetype'], 102);
        $newString = PutString($newString, $dbInfo['dbtype'], 104, 1);
        $newString = PutString($newString, $dbInfo['extra'], 105, 27);

        $status = FALSE;

        // Read the file.
        $fileString = ReadCfgFile($file);
        if ($fileString !== FALSE) {
            // Append the data to the end.
            $fileString = $fileString . $newString;

            // Write the file.
            $status = WriteCfgFile($file, $fileString);
        }

        // Return the write status.
        return $status;
    }


    /**
    * Va-Database.inc.php :: QuoteMessage()
    *
    * This function will take the passed message and add > to the beginning of each
    * line and truncate each line to 78 characters. This is used when quoting a
    * message a user is replying to.
    *
    * @param string $message
    * @return string
    */
    function QuoteMessage($message) {
        if (!is_string($message)) {
            $message = (string) $message;
        }

        $message = preg_replace("/\r\n|\n|\r/", "\r\n", StripBBSCodes($message));
        $message = explode("\r\n", $message);
        for ($i = 0; $i < count($message); $i++) {
            if (strlen(rtrim($message[$i])) >= 77) {
                $message[$i] = substr(rtrim($message[$i]), 0, 77);
            }
            $message[$i] = rtrim($message[$i]);
        }
        $message = "\r\n\r\n> " . implode("\r\n> ", $message) . "\r\n";

        return $message;
    }


    /**
    * Va-Database.inc.php :: RssAccess()
    *
    * This function will determine if the user has access clearance for the specified
    * database.
    *
    * @param array $dbInfo
    * @return boolean
    */
    function RssAccess($dbInfo) {
        if (!is_array($dbInfo)) {
            trigger_error(VA_Error(1));
        }

        if ((!isset($dbInfo)) ||
            (!isset($dbInfo['minreadsl']))) {
            return FALSE;
        }

        $classUserfileDat = new datUserfile;

        $user = $classUserfileDat->Get(0);

        // Set the default access value, false.
        $access = FALSE;

        // Check security level, age and the user access flags.
        if (($user['securitylevel'] >= $dbInfo['minreadsl']) &&
            ($user['age'] >= $dbInfo['agelimit']) &&
            (($dbInfo['accflags'] == '') ||
             (strpos($user['accflags'], $dbInfo['accflags']) !== FALSE))) {
            $access = TRUE;
        }

        // Return the access result.
        return $access;
    }


    /**
    * vadvDatabases :: SaveDb()
    *
    * Save information into a database.
    *
    * @param array $DatabaseInf
    * @param string $sFromUserNo
    * @param string $sToUserString
    * @param string $sMsgSubject
    * @param string $sMsgBody
    * @param string $sAttachedFile
    * @param integer $iThreadId
    * @param integer $iThreadNode
    * @param boolean $bSaveSent
    * @param integer $iSignature
    * @param string $sTagline
    * @return boolean
    */
    public function SaveDb($DatabaseInf,
                    $sFromUserNo,
                    $sToUserString,
                    $sMsgSubject,
                    $sMsgBody,
                    $sAttachedFile = '',
                    $iThreadId = 0,
                    $iThreadNode = 0,
                    $bSaveSent = FALSE,
                    $iSignature = 0,
                    $sTagline = '') {
        if (!is_string($sFromUserNo)) {
            $sFromUserNo = (string) $sFromUserNo;
        }
        if (!is_string($sToUserString)) {
            $sToUserString = (string) $sToUserString;
        }
        if (!is_string($sMsgSubject)) {
            $sMsgSubject = (string) $sMsgSubject;
        }
        if (!is_string($sMsgBody)) {
            $sMsgBody = (string) $sMsgBody;
        }
        if (!is_string($sAttachedFile)) {
            $sAttachedFile = (string) $sAttachedFile;
        }
        if (!is_int($iThreadId)) {
            $iThreadId = (integer) $iThreadId;
        }
        if (!is_int($iThreadNode)) {
            $iThreadNode = (integer) $iThreadNode;
        }
        if (!is_bool($bSaveSent)) {
            $bSaveSent = (boolean) $bSaveSent;
        }
        if (!is_string($sTagline)) {
            $sTagline = (string) $sTagline;
        }

        $classUserfileDat = new datUserfile;
        $classVadvUsers = new vadvUsers;

        // Check for database information.
        if (trim($DatabaseInf['filename']) == '') {
            return FALSE;
        }

        $bIsEmail = FALSE;
        $bIsFile = FALSE;
        $bIsMessage = FALSE;
        // Determine what type of database.
        switch (strtoupper($DatabaseInf['dbtype'])) {
            case 'E':
                $bIsEmail = TRUE;
                break;
            case 'F':
                $bIsFile = TRUE;
                break;
            case 'M':
                $bIsMessage = TRUE;
                break;
        }

        // Format required parameters.
        $sFromUserNo = trim(intval($sFromUserNo));
        $sToUserString = trim($sToUserString);
        $sMsgSubject = rtrim(substr($sMsgSubject, 0, 64));
        $sMsgBody = rtrim(wordwrap($sMsgBody, 79, "\r\n", 1)) . "\r\n";

        // Check user number.
        if (intval($sFromUserNo < 1) ||
            intval($sFromUserNo > $classVadvUsers->Count())) {
            return FALSE;
        }

        // Get from user information.
        $FromUserInf = $classUserfileDat->Get(intval($sFromUserNo));

        // Check for user configured tagline. (Macros)
        switch ($iSignature) {
            case 1:
                if (rtrim($FromUserInf['macro1']) != '')  {
                    $sMsgBody = $sMsgBody . "\r\n\r\n" . $FromUserInf['macro1'];
                }
                break;
            case 2:
                if (rtrim($FromUserInf['macro2']) != '')  {
                    $sMsgBody = $sMsgBody . "\r\n\r\n" . $FromUserInf['macro2'];
                }
                break;
        }

        // Add any email tagline.
        if ($bIsEmail) {
            $sTagline = trim($sTagline);
            if ($sTagline != '') {
            }
        }

        return TRUE;
    }


    /**
    * Va-Database.inc.php :: SearchMessagesFrom()
    *
    * This function will scan the current database for all messages from the
    * current user and store them into an array.
    *
    * @param array $dbInfo
    * @param integer $userNo
    * @return array
    */
    function SearchMessagesFrom($dbInfo,
                                $userNo) {
        if (!is_array($dbInfo)) {
            trigger_error(VA_Error(1));
        }
        if (!is_int($userNo)) {
            $userNo = (integer) $userNo;
        }

        $classNetworks = new vadvNetworks;

        if ($userNo == 0) {
            $msgFrom[1] = '';
            return $msgFrom;
        }

        // This function has been optimized for speed and does not use the pre-written functions above.
        // Set the filename.
        $file = $GLOBALS['VA_SYSPATHS']['db'] . '/' . $dbInfo['filename'];

        // Set the database filename.
        $datFile = $file . '.DAT';

        if (!file_exists($datFile)) {
            return FALSE;
        }

        $netInfo = $classNetworks->GetNetworkInfo();

        // Read the database file.
        $fileString = ReadCfgFile($datFile);

        // Counter.
        $cnt = 1;
        $msgFrom = array();

        for ($msgNo = 1; $msgNo <= $dbInfo['msgno']; $msgNo++) {
            $offset = 582 * ($msgNo - 1);
            $msgInfo['fromuserno'] = GetLongInt($fileString, 1 + $offset);
            if ($msgInfo['fromuserno'] == $userNo) {
                $msgInfo['fromnetid'] = GetInteger($fileString, 249 + $offset);
                if ((isset($netInfo[$msgInfo['fromnetid']]['netno'])) &&
                   ($msgInfo['fromnetid'] == $netInfo[$msgInfo['fromnetid']]['netno'])) {
                    $msgInfo['fromnetnode'] = GetString($fileString, 89 + $offset, 80);
                    if ($msgInfo['fromnetnode'] == $netInfo[$msgInfo['fromnetid']]['settings']['systemid']) {
                        // Add the message number to the array.
                        $msgFrom[$cnt] = $msgNo;

                        // Increase the counter by one.
                        $cnt++;
                    }
                }
            }
        }

        // Set the total number of messages.
        $msgFrom[0] = $cnt - 1;

        // Return the messages from this user.
        return $msgFrom;
    }


    /**
    * Va-Database.inc.php :: SearchMessagesTo()
    *
    * This function will scan the current database for all messages to the current
    * user and store them into an array.
    *
    * @param array $dbInfo
    * @param integer $userNo
    * @return array
    */
    function SearchMessagesTo($dbInfo,
                              $userNo) {
        if (!is_array($dbInfo)) {
            trigger_error(VA_Error(1));
        }
        if (!is_int($userNo)) {
            $userNo = (integer) $userNo;
        }

        $classNetworks = new vadvNetworks;

        if ($userNo == 0) {
            $msgTo[1] = '';
            return $msgTo;
        }

        // This function has been optimized for speed and does not use the pre-written functions above.
        // Set the filename.
        $file = $GLOBALS['VA_SYSPATHS']['db'] . '/' . $dbInfo['filename'];

        // Set the database filename.
        $datFile = $file . '.DAT';
        if (!file_exists($datFile)) {
            return FALSE;
        }

        $netInfo = $classNetworks->GetNetworkInfo();

        // Read the database file.
        $fileString = ReadCfgFile($datFile);

        $cnt = 1;
        $msgTo = array();

        for ($msgNo = 1; $msgNo <= $dbInfo['msgno']; $msgNo++) {
            $offset = 582 * ($msgNo - 1);
            $msgInfo['touserno'] = GetLongInt($fileString, 5 + $offset);
            if ($msgInfo['touserno'] == $userNo) {
                $msgInfo['tonetid'] = GetInteger($fileString, 251 + $offset);
                if ((isset($netInfo[$msgInfo['tonetid']]['netno'])) &&
                   ($msgInfo['tonetid'] == $netInfo[$msgInfo['tonetid']]['netno'])) {
                    $msgInfo['tonetnode'] = GetString($fileString, 169 + $offset, 80);
                    if ($msgInfo['tonetnode'] == $netInfo[$msgInfo['tonetid']]['settings']['systemid']) {
                        // Add the message number to the array.
                        $msgTo[$cnt] = $msgNo;

                        // Increase the counter by one.
                        $cnt++;
                    }
                }
            }
        }

        // Set the total number of messages.
        $msgTo[0] = $cnt - 1;

        // Return the messages from this user.
        return $msgTo;
    }


    /**
    * Va-Database.inc.php :: SetEmailInfo()
    *
    * This function will return the email database information in the same format
    * as dbInfo.
    *
    * @param integer $userNo
    * @return array
    */
    function SetEmailInfo($userNo) {
        if (!is_int($userNo)) {
            $userNo = (integer) $userNo;
        }

        $emailInfo['topicno'] = '';
        $emailInfo['topic'] = 'Email';
        $emailInfo['dbno'] = 0;
        $emailInfo['filename'] = 'EMAIL';
        if ($GLOBALS['CONFIG']['filehost'] != '') {
            $emailInfo['filepath'] = ConvertUnc($GLOBALS['VA_SYSPATHS']['sysop']);
        } else {
            $emailInfo['filepath'] = $GLOBALS['VA_SYSPATHS']['sysop'];
        }
        $emailInfo['desc'] = 'Personal Email';
        $emailInfo['private'] = 1;
        $emailInfo['dbgroup'] = '';
        $emailInfo['maxmsg'] = $GLOBALS['VA_MAIN']['mailholdtime'];
        $emailInfo['minreadsl'] = 0;
        $emailInfo['minwritesl'] = $GLOBALS['VA_MAIN']['emlothers'];
        $emailInfo['agelimit'] = 0;
        $emailInfo['accflags'] = 0;
        $emailInfo['tagline'] = 0;
        $emailInfo['dispname'] = 0;
        $emailInfo['randtitle'] = 0;
        $emailInfo['qwkconfno'] = 0;
        $emailInfo['moderator'] = 0;
        $emailInfo['validate'] = 0;
        $emailInfo['freqable'] = 0;
        $emailInfo['devicetype'] = 0;
        $emailInfo['dbtype'] = 'E';
        $emailInfo['extra'] = '';
        $emailInfo['msgno'] = $this->GetMessageCount($emailInfo);
        $emailInfo['msgcnt'] = $this->GetEmailCount($userNo);

        // If database does not exist, then create it.
        $file = $GLOBALS['VA_SYSPATHS']['db'] . '/' . $emailInfo['filename'] . '.DAT';
        if (!file_exists($file)) {
            WriteCfgFile($file, '');
        }

        // Return the email database information.
        return $emailInfo;
    }


    /**
    * Va-Database.inc.php :: SetOnelinerInfo()
    *
    * This function will return the oneliner database information in the same format
    * as dbInfo.
    *
    * @param integer $userNo
    * @return array
    */
    function SetOnelinerInfo($userNo = 0) {
        if (!is_int($userNo)) {
            $userNo = (integer) $userNo;
        }

        $onelinerInfo['topicno'] = '';
        $onelinerInfo['topic'] = 'Oneliners';
        $onelinerInfo['dbno'] = 0;
        $onelinerInfo['filename'] = 'ONELINE';
        $onelinerInfo['filepath'] = '';
        $onelinerInfo['desc'] = 'Oneliners';
        $onelinerInfo['private'] = 0;
        $onelinerInfo['dbgroup'] = '';
        $onelinerInfo['maxmsg'] = 0;
        $onelinerInfo['minreadsl'] = 0;
        $onelinerInfo['minwritesl'] = 0;
        $onelinerInfo['agelimit'] = 0;
        $onelinerInfo['accflags'] = 0;
        $onelinerInfo['tagline'] = 0;
        $onelinerInfo['dispname'] = 0;
        $onelinerInfo['randtitle'] = 0;
        $onelinerInfo['qwkconfno'] = 0;
        $onelinerInfo['moderator'] = 0;
        $onelinerInfo['validate'] = 0;
        $onelinerInfo['freqable'] = 0;
        $onelinerInfo['devicetype'] = 0;
        $onelinerInfo['dbtype'] = 'O';
        $onelinerInfo['extra'] = '';
        $onelinerInfo['msgno'] = $this->GetMessageCount($onelinerInfo);
        if ($userNo > 0) {
            $onelinerInfo['msgcnt'] = $this->GetOnelinerCount($userNo);
        }

        // If database does not exist, then create it.
        $file = $GLOBALS['VA_SYSPATHS']['db'] . '/' . $onelinerInfo['filename'] . '.DAT';
        if (!file_exists($file)) {
            WriteCfgFile($file, '');
        }

        // Return the oneliner database information.
        return $onelinerInfo;
    }


    /**
    * Va-Database.inc.php :: SetSentEmailInfo()
    *
    * This function will return the sent email database information in the same
    * format as dbInfo.
    *
    * @param integer $userNo
    * @return array
    */
    function SetSentEmailInfo($userNo) {
        if (!is_int($userNo)) {
            $userNo = (integer) $userNo;
        }

        $emailInfo['topicno'] = '';
        $emailInfo['topic'] = 'SentEmail';
        $emailInfo['dbno'] = 0;
        $emailInfo['filename'] = 'SNTEMAIL';
        if ($GLOBALS['CONFIG']['filehost'] != '') {
            $emailInfo['filepath'] = ConvertUnc($GLOBALS['VA_SYSPATHS']['sysop']);
        } else {
            $emailInfo['filepath'] = $GLOBALS['VA_SYSPATHS']['sysop'];
        }
        $emailInfo['desc'] = 'Sent Email';
        $emailInfo['private'] = 1;
        $emailInfo['dbgroup'] = '';
        $emailInfo['maxmsg'] = $GLOBALS['VA_MAIN']['mailholdtime'];
        $emailInfo['minreadsl'] = 0;
        $emailInfo['minwritesl'] = $GLOBALS['VA_MAIN']['emlothers'];
        $emailInfo['agelimit'] = 0;
        $emailInfo['accflags'] = 0;
        $emailInfo['tagline'] = 0;
        $emailInfo['dispname'] = 0;
        $emailInfo['randtitle'] = 0;
        $emailInfo['qwkconfno'] = 0;
        $emailInfo['moderator'] = 0;
        $emailInfo['validate'] = 0;
        $emailInfo['freqable'] = 0;
        $emailInfo['devicetype'] = 0;
        $emailInfo['dbtype'] = 'O';
        $emailInfo['extra'] = '';
        $emailInfo['msgno'] = $this->GetMessageCount($emailInfo);
        $emailInfo['msgcnt'] = $this->GetSentEmailCount($userNo);

        // If database does not exist, then create it.
        $file = $GLOBALS['VA_SYSPATHS']['db'] . '/' . $emailInfo['filename'] . '.DAT';
        if (!file_exists($file)) {
            WriteCfgFile($file, '');
        }

        // Return the email database information.
        return $emailInfo;
    }


    /**
    * Va-Database.inc.php :: SetUploadInfo()
    *
    * This function will return the upload database information in the same format
    * as dbInfo.
    *
    * @return array
    */
    function SetUploadInfo() {
        $uploadsInfo['topicno'] = '';
        $uploadsInfo['topic'] = 'Uploads';
        $uploadsInfo['dbno'] = 0;
        $uploadsInfo['filename'] = 'UPLOADS';
        if ($GLOBALS['CONFIG']['filehost'] != '') {
            $uploadsInfo['filepath'] = ConvertUnc($GLOBALS['VA_SYSPATHS']['sysop']);
        } else {
            $uploadsInfo['filepath'] = $GLOBALS['VA_SYSPATHS']['sysop'];
        }
        $uploadsInfo['desc'] = 'Temporary Upload DB';
        $uploadsInfo['private'] = 0;
        $uploadsInfo['dbgroup'] = '';
        $uploadsInfo['maxmsg'] = 0;
        $uploadsInfo['minreadsl'] = 250;
        $uploadsInfo['minwritesl'] = 0;
        $uploadsInfo['agelimit'] = 0;
        $uploadsInfo['accflags'] = 0;
        $uploadsInfo['tagline'] = 0;
        $uploadsInfo['dispname'] = 0;
        $uploadsInfo['randtitle'] = 0;
        $uploadsInfo['qwkconfno'] = 0;
        $uploadsInfo['moderator'] = 0;
        $uploadsInfo['validate'] = 0;
        $uploadsInfo['freqable'] = 0;
        $uploadsInfo['devicetype'] = 0;
        $uploadsInfo['dbtype'] = 'F';
        $uploadsInfo['extra'] = '';
        $uploadsInfo['msgno'] = $this->GetMessageCount($uploadsInfo);

        // If database does not exist, then create it.
        $file = $GLOBALS['VA_SYSPATHS']['db'] . '/' . $uploadsInfo['filename'] . '.DAT';
        if (!file_exists($file)) {
            WriteCfgFile($file, '');
        }

        // Return the oneliner database information.
        return $uploadsInfo;
    }


    /**
    * Va-Database.inc.php :: UpdateMessagePointers()
    *
    * This function will scan ALL message databases and update ALL message pointers
    * to the most current message. If $ignore is set to 1, it will obey the
    * join/ignore flags set by the user.
    *
    * @param integer $userNo
    * @param boolean $ignore
    */
    function UpdateMessagePointers($userNo,
                                   $ignore = 1) {
        if (!is_int($userNo)) {
            $userNo = (integer) $userNo;
        }
        if (!is_bool($ignore)) {
            $ignore = (boolean) $ignore;
        }

        $classUserfileDat = new datUserfile;

        $user = $classUserfileDat->Get($userNo);
        $dbString = ReadCfgFile($GLOBALS['VA_SYSPATHS']['main'] . '/DATABASE.CFG');

        // Get the accessible topics.
        $topics = $this->GetTopics('M');
        for ($i = 1; $i <= $topics[0]; $i++) {
            $dbno = $this->GetDbCount($topics[$i]);
            for ($j = 1; $j <= $dbno; $j++) {
                $dbInfo = $this->GetDbInfo($topics[$i], $j, $dbString);
                $file = $GLOBALS['VA_SYSPATHS']['db'] . '/' . $dbInfo['filename'] . '.QSC';
                $fileString = '';
                if (file_exists($file)) {
                    $fileString = ReadCfgFile($file);
                }
                if (!$ignore) {
                    if ($this->DbAccess($dbInfo, '', $user)) {
                        $this->WriteMessagePointer($dbInfo, $userNo, $dbInfo['msgno'], $fileString);
                    }
                } else {
                    if (($this->GetMessagePointer($dbInfo, $userNo, $fileString) < 32760) &&
                        ($this->DbAccess($dbInfo, '', $user))) {
                        $this->WriteMessagePointer($dbInfo, $userNo, $dbInfo['msgno'], $fileString);
                    }
                }
            }
        }
    }


    /**
    * Va-Database.inc.php :: WriteEmail()
    *
    * This function will write the email header information and the message text
    * to the proper files based on the passed database information. If a file is
    * attached to the email, then the filename must be in the 8.3 format. VADV does
    * not support long filenames.
    *
    * This function is to write only new email messages. For message area or files,
    * use the appropriate function. If this is a reply to a message, set $reply to
    * the message number of the original message.
    *
    * The toUserString is in the format of user@node*netnumber, for example:
    * to send an email to me, it would be 1@1*80. Local messages are given a network
    * number of 0.
    *
    * @param integer $fromUserNo
    * @param string $toUserString
    * @param string $msgSubject
    * @param string $msgBody
    * @param integer $reply
    * @param string $attFile
    * @param boolean $saveSent
    * @param string $attFile
    * @return mixed
    */
    function WriteEmail($fromUserNo,
                        $toUserString,
                        $msgSubject,
                        $msgBody,
                        $reply = 0,
                        $attFile = '',
                        $saveSent = 1) {
        if (!is_int($fromUserNo)) {
            $fromUserNo = (integer) $fromUserNo;
        }
        if (!is_string($toUserString)) {
            $toUserString = (string)  $toUserString;
        }
        if (!is_string($msgSubject)) {
            $msgSubject = (string)  $msgSubject;
        }
        if (!is_string($msgBody)) {
            $msgBody = (string)  $msgBody;
        }
        if (!is_int($reply)) {
            $reply = (integer) $reply;
        }
        if (!is_string($attFile)) {
            $attFile = (string)  $attFile;
        }
        if (!is_bool($saveSent)) {
            $saveSent = (boolean) $saveSent;
        }

        if ($fromUserNo < 1) {
            return FALSE;
        }

        $classNetworks = new vadvNetworks;
        $classStatVBBSDat = new datStatVBBS;
        $classUserfileDat = new datUserfile;
        $classVadvUsers = new vadvUsers;

        $toUserString = trim($toUserString);
        $msgSubject = rtrim(strip_tags($msgSubject));
        if (strlen($msgSubject) > 64) {
            $msgSubject = substr($msgSubject, 0, 64);
        }
        $msgBody = UnHtmlSpecialChars($msgBody);
        $msgBody = rtrim(wordwrap($msgBody, 79, "\r\n", 1));

        // If user auto-sig is set...
        if ($GLOBALS['USER_SETTINGS']['tagline'] > 0) {
            if ($GLOBALS['USER_SETTINGS']['tagline'] == 1) {
                $msgBody = $msgBody . "\r\n\r\n" . rtrim($GLOBALS['USER']['macro1']);
            } else {
                $msgBody = $msgBody . "\r\n\r\n" . rtrim($GLOBALS['USER']['macro2']);
            }
        }

        // Check for email tagline file.
        $file = $GLOBALS['VA_SYSPATHS']['text'] . '/EMAIL.TAG';
        if (file_exists($file)) {
            // Add the tagline if there.
            $msgBody = $msgBody . "\r\n\r\n" . ReadCfgFile($file);
        }

        $msgBody = rtrim($msgBody) . "\r\n";

        // Get the local network info.
        $networks = $classNetworks->GetNetworkInfo();
        $dbInfo = $this->SetEmailInfo($fromUserNo);
        $sentdbInfo = $this->SetSentEmailInfo($fromUserNo);

        // Setup file variables if a file was attached.
        if ((trim($attFile) != '') &&
            (file_exists($attFile))) {
            $attFilesize = filesize($attFile);
            $pos = strrpos($attFile, '/');
            if ($pos == 0) {
                $pos = strrpos($attFile, "\\");
            }
            if ($pos > 0) {
                $attFilename = substr($attFile, $pos + 1);
                if ($GLOBALS['CONFIG']['hostname'] != '') {
                    $attFilepath = ConvertUnc($attFile);
                } else {
                    $attFilepath = $attFile;
                }
            } else {
                $attFilesize = 0;
                $attFilename = '';
                $attFilepath = '';
            }
        } else {
            $attFilesize = 0;
            $attFilename = '';
            $attFilepath = '';
        }

        // Break down the toUserString variable.
        $atpos = strpos($toUserString, '@');
        $aspos = strrpos($toUserString, '*');
        $toUserNo = trim(substr($toUserString, 0, $atpos));
        $toNetNode = trim(substr($toUserString, $atpos + 1, $aspos - $atpos - 1));
        $toNetId = trim(substr($toUserString, $aspos + 1));

        // Check to see if it a local net id.
        if (($toNetId == 1) &&
            ($toNetNode == $networks[1]['settings']['systemid'])) {
            $toNetNode = 0;
            $toNetId = 0;
        }

        // If a local email, check to see if the user exists.
        if ($toNetId == 0) {
            if (is_numeric($toUserNo)) {
                $toUser = $classUserfileDat->Get($toUserNo);
            } else {
                $toUser = $classVadvUsers->FindUser($toUserNo);
                if ($toUser == 0) {
                    die('<br /><h4>The user specified is invalid. Please press the BACK button on your browser.</h4>');
                }
                $toUserNo = $toUser;
                $toUser = $classUserfileDat->Get($toUser);
            }
            // Check if the user has their email forwarded.
            if ($toUser['mailfwdflag'] == 1) {
                // If forwarded to another local user...
                if (($toUser['mailfwdnetno'] == 1) &&
                    ($toUser['mailfwdnode'] == $networks[1]['settings']['systemid'])) {
                    $toUserNo = $toUser['mailfwduserno'];
                    $toUser = $classUserfileDat->Get($toUserNo);
                } else {
                    // If forwarded to another location...
                    if ($toUser['mailfwduserno'] == 0) {
                        // If the to user no field is blank...
                        $toUserNo = $toUser['mailfwdhandle'];
                    } else {
                        // If the to user no field is not blank...
                        $toUserNo = $toUser['mailfwduserno'];
                    }
                    $toNetNode = $toUser['mailfwdnode'];
                    $toNetId = $toUser['mailfwdnetno'];
                }
            }
        }

        // Set the toUserHandle variable.
        if ($toNetId == 0) {
            $toUserHandle = $toUser['handle'];
            $toNetNode = $networks[1]['settings']['systemid'];
        } else {
            if (is_numeric($toUserNo)) {
                $toUserHandle = '';
            } else {
                $toUserHandle = $toUserNo;
                $toUserNo = 0;
            }
        }

        // Set the filename.
        $file = $GLOBALS['VA_SYSPATHS']['db'] . '/' . $dbInfo['filename'];

        // Set the bin filename.
        $binFile = $file . '.BIN';

        // Set the database filename.
        $datFile = $file . '.DAT';

        // Set the sent filename.
        $sentfile = $GLOBALS['VA_SYSPATHS']['db'] . '/' . $sentdbInfo['filename'];

        // Set the bin filename.
        $sentbinFile = $sentfile . '.BIN';

        // Set the database filename.
        $sentdatFile = $sentfile . '.DAT';

        // Get the 'from' user information and update it.
        $fileString = $classUserfileDat->Read();
        $fromUser = $classUserfileDat->Get($fromUserNo, $fileString);
        $fromUser['totalemail']++;
        $fileString = $classUserfileDat->Put($fileString, $fromUserNo, $fromUser);
        $status = $classUserfileDat->Write($fileString);

        // Set the $fromUserHandle variable.
        $fromUserHandle = $fromUser['handle'];
        if (($toNetId != 0) &&
            ($networks[$toNetId]['nettype'] == 4)) {
            if ($networks[$toNetId]['realname'] == 1) {
                $fromUserHandle = $fromUser['realname'];
            }
        }

        // Update the BBS stats.
        $fileString = $classStatVBBSDat->Read();
        $stats = $classStatVBBSDat->Get($fileString);
        $stats['emailstotal']++;
        $stats['emailstoday']++;
        $fileString = $classStatVBBSDat->Put($fileString, $stats);
        $status = $classStatVBBSDat->Write($fileString);

        if ($toNetId != 0) {
            $msgpos = 0;
        } else {
            if (!file_exists($binFile)) {
                $msgpos = 1;
            } else {
                // Get the message starting position.
                $msgpos = filesize($binFile) + 1;
            }
        }

        // Get the message length.
        $msglen = strlen($msgBody);
        if ($msglen > 30000) {
            $msgBody = substr($msgBody, 0, 30000);
            $msglen = strlen($msgBody);
        }

        // Add spacing of 582 bytes of null chars.
        // Create the new blank string.
        $newString = str_pad('', 582, chr(0));

        $newString = PutLongInt($newString, $fromUserNo, 1);
        $newString = PutLongInt($newString, $toUserNo, 5);
        $newString = PutString($newString, $fromUserHandle, 9, 40);
        $newString = PutString($newString, $toUserHandle, 49, 40);
        $newString = PutString($newString, $networks[1]['settings']['systemid'], 89, 80);
        $newString = PutString($newString, $toNetNode, 169, 80);
        $newString = PutInteger($newString, 1, 249);
        if ($toNetId == 0) {
            $newString = PutInteger($newString, 1, 251);
        } else {
            $newString = PutInteger($newString, $toNetId, 251);
        }
        $newString = PutLongInt($newString, lTime(), 253);
        $newString = PutLongInt($newString, $this->WriteTotalMessageCount(), 257);
        $newString = PutLongInt($newString, 0, 261);
        $newString = PutLongInt($newString, 0, 265);
        $newString = PutString($newString, $msgSubject, 269, 64);
        $newString = PutInteger($newString, 0, 333);
        $newString = PutString($newString, date("D M d, Y H:i:s O"), 335, 40);
        $newString = PutLongInt($newString, 0, 375);
        $newString = PutLongInt($newString, $attFilesize, 379);
        $newString = PutString($newString, strtoupper($attFilename), 383, 12);
        $newString = PutString($newString, strtoupper($attFilepath), 395, 64);
        $newString = PutLongInt($newString, 0, 459);
        $newString = PutLongInt($newString, 0, 463);
        $newString = PutLongInt($newString, 0, 467);
        $newString = PutString($newString, $dbInfo['filename'], 471, 8);
        $newString = PutLongInt($newString, $msgpos, 479);
        $newString = PutLongInt($newString, $msglen, 483);
        $newString = PutInteger($newString, $toNetId, 487);
        $newString = PutInteger($newString, 0, 489);
        $newString = PutLongInt($newString, 0, 491);
        $newString = PutString($newString, '', 495, 24);
        $newString = PutString($newString, '', 519, 64);

        // If a local message, write it to the database, if not, write it to spool.net.
        if ($toNetId == 0) {
            // Update the .DAT file (and update dbseq.dat).
            // Read the file.
            $fileString = ReadCfgFile($datFile);
            if ($fileString === FALSE) {
                return FALSE;
            }

            // Error correction...
            $fslen = strlen($fileString);
            $fsnum = $fslen / 582;
            if (is_int($fsnum)) {
                // Append the new data to the original file string.
                $fileString = $fileString . $newString;
            } else {
                $fileString = substr($fileString, 0, floor($fsnum) * 582) . $newString;
            }

            // Write the new data file.
            $status = WriteCfgFile($datFile, $fileString);

            // Add the message to the .BIN file.
            // Open the file.
            $fp = fopen($binFile, 'a');
            // Lock the file.
            flock($fp, LOCK_EX);
            // Write the message.
            $status = fwrite($fp, $msgBody);
            // Unlock the file.
            flock($fp, LOCK_UN);
            // Close the file.
            fclose($fp);

            // Send a one liner to the user if online.
            $oneliner = $fromUser['handle'] . ' just sent you some email!';
            SendOneLiner($oneliner, $toUserNo);
        } else {
            // Create validation file or spool.net file based on db settings.
            // Create network spool file.
            $file = $GLOBALS['VA_SYSPATHS']['data'] . '/SPOOL.NET';

            // Open the file.
            $fp = fopen($file, 'a');
            // Lock the file.
            flock($fp, LOCK_EX);
            // Write the message.
            $status = fwrite($fp, $newString . $msgBody);
            // Unlock the file.
            flock($fp, LOCK_UN);
            // Close the file.
            fclose($fp);
        }

        if ($saveSent) {
            // Setup the sent email database entry.
            if (!file_exists($sentbinFile)) {
                $msgpos = 1;
            } else {
                // Get the message starting position.
                $msgpos = filesize($sentbinFile) + 1;
            }
            $newString = PutString($newString, $sentdbInfo['filename'], 471, 8);
            $newString = PutLongInt($newString, 1, 375);
            $newString = PutLongInt($newString, 0, 379);
            $newString = PutString($newString, '', 383, 12);
            $newString = PutString($newString, '', 395, 64);
            $newString = PutLongInt($newString, $msgpos, 479);

            // Save the email into the sent email database, so the user can retrieve it at a later time.
            // Read the file.
            $fileString = ReadCfgFile($sentdatFile);
            if ($fileString === FALSE) {
                return FALSE;
            }

            // Error correction...
            $fslen = strlen($fileString);
            $fsnum = $fslen / 582;
            if (is_int($fsnum)) {
                // Append the new data to the original file string.
                $fileString = $fileString . $newString;
            } else {
                $fileString = substr($fileString, 0, intval($fsnum) * 582) . $newString;
            }

            // Write the new data file.
            WriteCfgFile($sentdatFile, $fileString);

            // Add the message to the .BIN file.
            // Open the file.
            $fp = fopen($sentbinFile, 'a');
            // Lock the file.
            flock($fp, LOCK_EX);
            // Write the message.
            $status = fwrite($fp, $msgBody);
            // Unlock the file.
            flock($fp, LOCK_UN);
            // Close the file.
            fclose($fp);
        }

        // Return the write status.
        return $status;
    }


    /**
    * Va-Database.inc.php :: WriteFile()
    *
    * This function will write the file header information and the file description
    * to the proper files based on the passed database information. $attFile should
    * be a full path to the file and the filename should be in the 8.3 format. VADV
    * does not support long filenames.
    *
    * This function is to write only new file area entries. For emails or messages,
    * use the appropriate function.
    *
    * @param array $dbInfo
    * @param integer $fromUserNo
    * @param string $fileDesc
    * @param string $fileExtDesc
    * @param string $attFile
    * @return mixed
    */
    function WriteFile($dbInfo,
                       $fromUserNo,
                       $fileDesc,
                       $fileExtDesc,
                       $attFile) {
        if (!is_array($dbInfo)) {
            trigger_error(VA_Error(1));
        }
        if (!is_int($fromUserNo)) {
            $fromUserNo = (integer) $fromUserNo;
        }
        if (!is_string($fileDesc)) {
            $fileDesc = (string)  $fileDesc;
        }
        if (!is_string($fileExtDesc)) {
            $fileExtDesc = (string)  $fileExtDesc;
        }
        if (!is_string($attFile)) {
            $attFile = (string)  $attFile;
        }

        if ($fromUserNo < 1) {
            return FALSE;
        }

        $classNetworks = new vadvNetworks;
        $classStatVBBSDat = new datStatVBBS;
        $classUserfileDat = new datUserfile;

        $fileDesc = rtrim(strip_tags($fileDesc));
        if (strlen($fileDesc) > 64) {
            $fileDesc = substr($fileDesc, 0, 64);
        }
        $fileExtDesc = UnHtmlSpecialChars($fileExtDesc);
        $fileExtDesc = rtrim(wordwrap($fileExtDesc, 79, "\r\n", 1));

        // If tagline is set...
        if ($dbInfo['tagline'] != 0) {
            // Look for the file...
            $file = $GLOBALS['VA_SYSPATHS']['text'] . '/TAGLINE.' . $dbInfo['tagline'];
            if (file_exists($file)) {
                // Append the file if found.
                $fileExtDesc = $fileExtDesc . "\r\n\r\n" . ReadCfgFile($file);
            }
        }

        // Setup file variables if a file was attached.
        if ((trim($attFile) != '') &&
            (file_exists($attFile))) {
            $attFilesize = filesize($attFile);
            $pos = strrpos($attFile, '/');
            if ($pos == 0) {
                $pos = strrpos($attFile, "\\");
            }
            if ($pos > 0) {
                $attFilename = substr($attFile, $pos + 1);
            } else {
                $attFilesize = 0;
                $attFilename = '';
            }
        } else {
            $attFilesize = 0;
            $attFilename = '';
        }

        // Set the filename.
        $file = $GLOBALS['VA_SYSPATHS']['db'] . '/' . $dbInfo['filename'];
        // Set the bin filename.
        $binFile = $file . '.BIN';
        // Set the database filename.
        $datFile = $file . '.DAT';

        // Get the 'from' user information and update it.
        $fileString = $classUserfileDat->Read();
        $fromUser = $classUserfileDat->Get($fromUserNo, $fileString);
        $fromUser['uplfiles']++;
        $uplKbCredit = intval($attFilesize / 1024) * $GLOBALS['VA_MAIN']['credperuplk'];
        // Award credits for posting.
        $fromUser['credits'] = $fromUser['credits'] + $uplKbCredit;
        $fileString = $classUserfileDat->Put($fileString, $fromUserNo, $fromUser);
        $status = $classUserfileDat->Write($fileString);

        // Update the BBS stats.
        $fileString = $classStatVBBSDat->Read();
        $stats = $classStatVBBSDat->Get($fileString);
        $stats['uploadstotal']++;
        $stats['uploadstoday']++;
        $fileString = $classStatVBBSDat->Put($fileString, $stats);
        $status = $classStatVBBSDat->Write($fileString);

        if (!file_exists($binFile)) {
            $msgpos = 1;
        } else {
            // Get the message starting position.
            $msgpos = filesize($binFile) + 1;
        }

        // Get the message length.
        $msglen = strlen($fileExtDesc);
        if ($msglen > 30000) {
            $fileExtDesc = substr($fileExtDesc, 0, 30000);
            $msglen = strlen($fileExtDesc);
        }

        // Update the .DAT file (and update dbseq.dat).
        // Add spacing of 582 bytes of null chars.
        // Create the new blank string.
        $newString = str_pad('', 582, chr(0));

        // Get the local network info.
        $networks = $classNetworks->GetNetworkInfo();

        // Check for special database settings.
        if ($dbInfo['dispname'] == 1) {
            $fromHandle = $fromUser['realname'];
        } else if ($dbInfo['dispname'] == 2) {
            $fromHandle = 'Anonymous';
        } else {
            $fromHandle = $fromUser['handle'];
        }

        $newString = PutLongInt($newString, $fromUserNo, 1);
        $newString = PutLongInt($newString, 0, 5);
        $newString = PutString($newString, $fromHandle, 9, 40);
        $newString = PutString($newString, '', 49, 40);
        $newString = PutString($newString, $networks[1]['settings']['systemid'], 89, 80);
        $newString = PutString($newString, '', 169, 80);
        $newString = PutInteger($newString, 1, 249);
        $newString = PutInteger($newString, 1, 251);
        $newString = PutLongInt($newString, lTime(), 253);
        $newString = PutLongInt($newString, $this->WriteTotalMessageCount(), 257);
        $newString = PutLongInt($newString, 0, 261);
        $newString = PutLongInt($newString, 0, 265);
        $newString = PutString($newString, $fileDesc, 269, 64);
        $newString = PutInteger($newString, 0, 333);
        $newString = PutString($newString, date("D M d, Y H:i:s O"), 335, 40);
        $newString = PutLongInt($newString, 0, 375);
        $newString = PutLongInt($newString, $attFilesize, 379);
        $newString = PutString($newString, strtoupper($attFilename), 383, 12);
        $newString = PutString($newString, '', 395, 64);
        $newString = PutLongInt($newString, 0, 459);
        $newString = PutLongInt($newString, 0, 463);
        $newString = PutLongInt($newString, 0, 467);
        $newString = PutString($newString, $dbInfo['filename'], 471, 8);
        $newString = PutLongInt($newString, $msgpos, 479);
        $newString = PutLongInt($newString, $msglen, 483);
        $newString = PutInteger($newString, 0, 487);
        $newString = PutInteger($newString, 0, 489);
        $newString = PutLongInt($newString, 0, 491);
        $newString = PutString($newString, '', 495, 24);
        $newString = PutString($newString, '', 519, 64);

        // Read the file.
        $fileString = ReadCfgFile($datFile);
        if ($fileString === FALSE) {
            return FALSE;
        }

        // Error correction...
        $fslen = strlen($fileString);
        $fsnum = $fslen / 582;
        if (is_int($fsnum)) {
            // Append the new data to the original file string.
            $fileString = $fileString . $newString;
        } else {
            $fileString = substr($fileString, 0, floor($fsnum) * 582) . $newString;
        }

        // Write the new data file.
        $status = WriteCfgFile($datFile, $fileString);

        // Add the message to the .BIN file.
        // Open the file.
        $fp = fopen($binFile, 'a');
        // Lock the file.
        flock($fp, LOCK_EX);
        // Write the message.
        $status = fwrite($fp, $fileExtDesc);
        // Unlock the file.
        flock($fp, LOCK_UN);
        // Close the file.
        fclose($fp);

        // Create validation file or spool.net file based on db settings.
        if ($dbInfo['validate'] == 1) {
            // Create local post validation file.
            $file = $GLOBALS['VA_SYSPATHS']['data'] . '/VALIDATE.0';
        } else {
            // Create network spool file.
            $file = $GLOBALS['VA_SYSPATHS']['data'] . '/SPOOL.NET';
        }
        // Open the file.
        $fp = fopen($file, 'a');
        // Lock the file.
        flock($fp, LOCK_EX);
        // Write the message.
        $status = fwrite($fp, $newString . $fileExtDesc);
        // Unlock the file.
        flock($fp, LOCK_UN);
        // Close the file.
        fclose($fp);

        // Return the write status.
        return $status;
    }


    /**
    * Va-Database.inc.php :: WriteMessage()
    *
    * This function will write the message header information and the message text
    * to the proper files based on the passed database information.
    *
    * This function is to write only new message area messages. For emails or files,
    * use the appropriate function. If this is a reply to a message, set $reply to
    * the message number of the original message.
    *
    * @param array $dbInfo
    * @param integer $fromUserNo
    * @param string $msgSubject
    * @param string $msgBody
    * @param integer $reply
    * @return mixed
    */
    function WriteMessage($dbInfo,
                          $fromUserNo,
                          $msgSubject,
                          $msgBody,
                          $reply = 0) {
        if (!is_array($dbInfo)) {
            trigger_error(VA_Error(1));
        }
        if (!is_int($fromUserNo)) {
            $fromUserNo = (integer) $fromUserNo;
        }
        if (!is_string($msgSubject)) {
            $msgSubject = (string)  $msgSubject;
        }
        if (!is_string($msgBody)) {
            $msgBody = (string)  $msgBody;
        }
        if (!is_int($reply)) {
            $reply = (integer) $reply;
        }

        if ($fromUserNo < 1) {
            return FALSE;
        }

        $classNetworks = new vadvNetworks;
        $classStatVBBSDat = new datStatVBBS;
        $classUserfileDat = new datUserfile;

        $msgSubject = rtrim(strip_tags($msgSubject));
        if (strlen($msgSubject) > 64) {
            $msgSubject = substr($msgSubject, 0, 64);
        }
        $msgBody = UnHtmlSpecialChars($msgBody);
        $msgBody = rtrim(wordwrap($msgBody, 79, "\r\n", 1));

        // If user auto-sig is set...
        if ($GLOBALS['USER_SETTINGS']['tagline'] > 0) {
            if ($GLOBALS['USER_SETTINGS']['tagline'] == 1) {
                $msgBody = $msgBody . "\r\n\r\n" . rtrim($GLOBALS['USER']['macro1']);
            } else {
                $msgBody = $msgBody . "\r\n\r\n" . rtrim($GLOBALS['USER']['macro2']);
            }
        }
        // If tagline is set...
        if ($dbInfo['tagline'] > 0) {
            // Look for the file...
            $file = $GLOBALS['VA_SYSPATHS']['text'] . '/TAGLINE.' . $dbInfo['tagline'];
            if (file_exists($file)) {
                if ($dbInfo['tagline'] < 990) {
                    // Regular tagline...
                    // Append the file if found.
                    $msgBody = $msgBody . "\r\n\r\n" . ReadCfgFile($file);
                } else {
                    // Random tagline...
                    // Read the file into memory.
                    $taglineString = ReadCfgFile($file);
                    // Cleanup the file.
                    $taglineString = mstr_replace("\r\n\r\n", "\r\n", trim($taglineString));
                    // Read the file into an array.
                    $taglineArray = explode("\r\n", $taglineString);
                    if (count($taglineArray) > 0) {
                        // Get the random tagline number.
                        $rndTagline = rand(0, count($taglineArray) - 1);
                        // Append the tagline.
                        $msgBody = $msgBody . "\r\n\r\n" . trim($taglineArray[$rndTagline]);
                    }
                }
            }
        }

        $msgBody = rtrim($msgBody) . "\r\n";

        // Set the filename.
        $file = $GLOBALS['VA_SYSPATHS']['db'] . '/' . $dbInfo['filename'];
        // Set the bin filename.
        $binFile = $file . '.BIN';
        // Set the database filename.
        $datFile = $file . '.DAT';

        // Get the 'from' user information and update it.
        $fileString = $classUserfileDat->Read();
        $fromUser = $classUserfileDat->Get($fromUserNo, $fileString);
        $fromUser['totalposts']++;
        // Award credits for posting.
        $fromUser['credits'] = $fromUser['credits'] + $GLOBALS['VA_MAIN']['credperpost'];
        $fileString = $classUserfileDat->Put($fileString, $fromUserNo, $fromUser);
        $status = $classUserfileDat->Write($fileString);

        // Update the BBS stats.
        $fileString = $classStatVBBSDat->Read();
        $stats = $classStatVBBSDat->Get($fileString);
        $stats['poststotal']++;
        $stats['poststoday']++;
        $fileString = $classStatVBBSDat->Put($fileString, $stats);
        $status = $classStatVBBSDat->Write($fileString);

        if (!file_exists($binFile)) {
            $msgpos = 1;
        } else {
            // Get the message starting position.
            $msgpos = filesize($binFile) + 1;
        }

        // Get the message length.
        $msglen = strlen($msgBody);
        if ($msglen > 30000) {
            $msgBody = substr($msgBody, 0, 30000);
            $msglen = strlen($msgBody);
        }

        // Update the .DAT file (and update dbseq.dat).
        // Add spacing of 582 bytes of null chars.
        // Create the new blank string.
        $newString = str_pad('', 582, chr(0));
        // Get the local network info.
        $networks = $classNetworks->GetNetworkInfo();

        // Check for special database settings.
        if ($dbInfo['dispname'] == 1) {
            $fromHandle = $fromUser['realname'];
        } else if ($dbInfo['dispname'] == 2) {
            $fromHandle = 'Anonymous';
        } else {
            $fromHandle = $fromUser['handle'];
        }

        // Check if a reply and set the variables accordingly.
        if ($reply > 0) {
            $origMessage = $this->GetMessageInfo($dbInfo, $reply);
            $toUserNo = $origMessage['fromuserno'];
            $toUser = $origMessage['fromhandle'];
            $toNetNode = $origMessage['fromnetnode'];
            $toNetId = $origMessage['fromnetid'];
            $threadid = $origMessage['sysmsgno'];
            $threadnode = $origMessage['fromnetnode'];
        } else {
            $toUserNo = 0;
            $toUser = '';
            $toNetNode = '';
            $toNetId = 0;
            $threadid = 0;
            $threadnode = 0;
        }
        $newString = PutLongInt($newString, $fromUserNo, 1);
        $newString = PutLongInt($newString, $toUserNo, 5);
        $newString = PutString($newString, $fromHandle, 9, 40);
        $newString = PutString($newString, $toUser, 49, 40);
        $newString = PutString($newString, $networks[1]['settings']['systemid'], 89, 80);
        $newString = PutString($newString, $toNetNode, 169, 80);
        $newString = PutInteger($newString, 1, 249);
        $newString = PutInteger($newString, $toNetId, 251);
        $newString = PutLongInt($newString, lTime(), 253);
        $newString = PutLongInt($newString, $this->WriteTotalMessageCount(), 257);
        $newString = PutLongInt($newString, $threadid, 261);
        $newString = PutLongInt($newString, $threadnode, 265);
        $newString = PutString($newString, $msgSubject, 269, 64);
        $newString = PutInteger($newString, 0, 333);
        $newString = PutString($newString, date("D M d, Y H:i:s O"), 335, 40);
        $newString = PutLongInt($newString, 0, 375);
        $newString = PutLongInt($newString, 0, 379);
        $newString = PutString($newString, '', 383, 12);
        $newString = PutString($newString, '', 395, 64);
        $newString = PutLongInt($newString, 0, 459);
        $newString = PutLongInt($newString, 0, 463);
        $newString = PutLongInt($newString, 0, 467);
        $newString = PutString($newString, $dbInfo['filename'], 471, 8);
        $newString = PutLongInt($newString, $msgpos, 479);
        $newString = PutLongInt($newString, $msglen, 483);
        if ($reply > 0) {
            $newString = PutInteger($newString, $toNetId, 487);
        } else {
            $newString = PutInteger($newString, 0, 487);
        }
        $newString = PutInteger($newString, 0, 489);
        $newString = PutLongInt($newString, 0, 491);
        $newString = PutString($newString, '', 495, 24);
        $newString = PutString($newString, '', 519, 64);

        // Read the file.
        $fileString = ReadCfgFile($datFile);
        if ($fileString === FALSE) {
            return FALSE;
        }

        // Error correction...
        $fslen = strlen($fileString);
        $fsnum = $fslen / 582;
        if (is_int($fsnum)) {
            // Append the new data to the original file string.
            $fileString = $fileString . $newString;
        } else {
            $fileString = substr($fileString, 0, floor($fsnum) * 582) . $newString;
        }

        // Write the new data file.
        $status = WriteCfgFile($datFile, $fileString);

        // Add the message to the .BIN file.
        // Open the file.
        $fp = fopen($binFile, 'a');
        // Lock the file.
        flock($fp, LOCK_EX);
        // Write the message.
        $status = fwrite($fp, $msgBody);
        // Unlock the file.
        flock($fp, LOCK_UN);
        // Close the file.
        fclose($fp);

        // Create validation file or spool.net file based on db settings.
        if ($dbInfo['validate'] == 1) {
            // Create local post validation file.
            $file = $GLOBALS['VA_SYSPATHS']['data'] . '/VALIDATE.0';
        } else {
            // Create network spool file.
            $file = $GLOBALS['VA_SYSPATHS']['data'] . '/SPOOL.NET';
        }
        // Open the file.
        $fp = fopen($file, 'a');
        // Lock the file.
        flock($fp, LOCK_EX);
        // Write the message.
        $status = fwrite($fp, $newString . $msgBody);
        // Unlock the file.
        flock($fp, LOCK_UN);
        // Close the file.
        fclose($fp);

        // Return the write
        return $status;
    }


    /**
    * Va-Database.inc.php :: WriteMessageInfo()
    *
    * This function will allow you to write new message information into the
    * corresponding .DAT file. You cannot change the creation date.
    *
    * @param array $dbInfo
    * @param integer $msgNo
    * @param array $configVariable
    * @return mixed
    */
    function WriteMessageInfo($dbInfo,
                              $msgNo,
                              $configVariable) {
        if (!is_array($dbInfo)) {
            trigger_error(VA_Error(1));
        }
        if (!is_int($msgNo)) {
            $msgNo = (integer) $msgNo;
        }
        if (!is_array($configVariable)) {
            trigger_error(VA_Error(1));
        }

        if ($msgNo < 1) {
            return FALSE;
        }
        if ($msgNo > $dbInfo['msgno']) {
            return FALSE;
        }

        // Set the filename.
        $file = $GLOBALS['VA_SYSPATHS']['db'] . '/' . $dbInfo['filename'];

        // Set the database filename.
        $datFile = $file . '.DAT';
        if (!file_exists($datFile)) {
            return FALSE;
        }

        // Read the file.
        $fileString = ReadCfgFile($datFile);
        if ($fileString === FALSE) {
            return FALSE;
        }

        $offset = 582 * ($msgNo - 1);
        if (isset($configVariable['fromuserno'])) {
            $fileString = PutLongInt($fileString, $configVariable['fromuserno'], 1 + $offset);
        }
        if (isset($configVariable['touserno'])) {
            $fileString = PutLongInt($fileString, $configVariable['touserno'], 5 + $offset);
        }
        if (isset($configVariable['fromhandle'])) {
            $fileString = PutString($fileString, $configVariable['fromhandle'], 9 + $offset, 40);
        }
        if (isset($configVariable['tohandle'])) {
            $fileString = PutString($fileString, $configVariable['tohandle'], 49 + $offset, 40);
        }
        if (isset($configVariable['fromnetnode'])) {
            $fileString = PutString($fileString, $configVariable['fromnetnode'], 89 + $offset, 80);
        }
        if (isset($configVariable['tonetnode'])) {
            $fileString = PutString($fileString, $configVariable['tonetnode'], 169 + $offset, 80);
        }
        if (isset($configVariable['fromnetid'])) {
            $fileString = PutInteger($fileString, $configVariable['fromnetid'], 249 + $offset);
        }
        if (isset($configVariable['tonetid'])) {
            $fileString = PutInteger($fileString, $configVariable['tonetid'], 251 + $offset);
        }
        if (isset($configVariable['sysmsgno'])) {
            $fileString = PutLongInt($fileString, $configVariable['sysmsgno'], 257 + $offset);
        }
        if (isset($configVariable['threadbk'])) {
            $fileString = PutLongInt($fileString, $configVariable['threadbk'], 261 + $offset);
        }
        if (isset($configVariable['threadbkn'])) {
            $fileString = PutLongInt($fileString, $configVariable['threadbkn'], 265 + $offset);
        }
        if (isset($configVariable['subject'])) {
            $fileString = PutString($fileString, $configVariable['subject'], 269 + $offset, 64);
        }
        if (isset($configVariable['deleted'])) {
            $fileString = PutInteger($fileString, $configVariable['deleted'], 333 + $offset);
        }
        if (isset($configVariable['offline'])) {
            $fileString = PutLongInt($fileString, $configVariable['offline'], 375 + $offset);
        }
        if (isset($configVariable['attfilesize'])) {
            $fileString = PutLongInt($fileString, $configVariable['attfilesize'], 379 + $offset);
        }
        if (isset($configVariable['attfilename'])) {
            $fileString = PutString($fileString, $configVariable['attfilename'], 383 + $offset, 12);
        }
        if (isset($configVariable['attfilepath'])) {
            $fileString = PutString($fileString, $configVariable['attfilepath'], 395 + $offset, 64);
        }
        if (isset($configVariable['lib'])) {
            $fileString = PutLongInt($fileString, $configVariable['lib'], 459 + $offset);
        }
        if (isset($configVariable['downloadno'])) {
            $fileString = PutLongInt($fileString, $configVariable['downloadno'], 463 + $offset);
        }
        if (isset($configVariable['dbid'])) {
            $fileString = PutLongInt($fileString, $configVariable['dbid'], 467 + $offset);
        }
        if (isset($configVariable['filename'])) {
            $fileString = PutString($fileString, $configVariable['filename'], 471 + $offset, 8);
        }
        if (isset($configVariable['msgpos'])) {
            $fileString = PutLongInt($fileString, $configVariable['msgpos'], 479 + $offset);
        }
        if (isset($configVariable['msglength'])) {
            $fileString = PutLongInt($fileString, $configVariable['msglength'], 483 + $offset);
        }
        if (isset($configVariable['originnet'])) {
            $fileString = PutInteger($fileString, $configVariable['originnet'], 487 + $offset);
        }
        if (isset($configVariable['fidoflags'])) {
            $fileString = PutInteger($fileString, $configVariable['fidoflags'], 489 + $offset);
        }
        if (isset($configVariable['originnode'])) {
            $fileString = PutLongInt($fileString, $configVariable['originnode'], 491 + $offset);
        }
        if (isset($configVariable['reserved'])) {
            $fileString = PutString($fileString, $configVariable['reserved'], 495 + $offset, 24);
        }
        if (isset($configVariable['extra'])) {
            $fileString = PutString($fileString, $configVariable['extra'], 519 + $offset, 64);
        }

        $status = WriteCfgFile($datFile, $fileString);

        // Return the result status.
        return $status;

    }


    /**
    * Va-Database.inc.php :: WriteMessagePointer()
    *
    * This function will update the message pointer file for the specified database
    * with either the supplied message number, or if no message number is passed,
    * then it will increase the pointer by one.
    *
    * @param array $dbInfo
    * @param integer $userNo
    * @param integer $msgNo
    * @return integer
    */
    function WriteMessagePointer($dbInfo,
                                 $userNo,
                                 $msgNo = 0,
                                 $fileString = '') {
        if (!is_array($dbInfo)) {
            trigger_error(VA_Error(1));
        }
        if (!is_int($userNo)) {
            $userNo = (integer) $userNo;
        }
        if (!is_int($msgNo)) {
            $msgNo = (integer) $msgNo;
        }
        if (!is_string($fileString)) {
            $fileString = (string) $fileString;
        }

        if ($userNo < 1) {
            return 32760;
        }

        $file = $GLOBALS['VA_SYSPATHS']['db'] . '/' . $dbInfo['filename'] . '.QSC';
        if ($fileString == '') {
            if ((!file_exists($file)) ||
                (filesize($file) == 0)) {
                $fileString = str_pad('', $userNo * 4, chr(0));
            } else {
                $fileString = ReadCfgFile($file);
            }
        }

        $offset = ($userNo - 1) * 4;
        $pointer = GetLongInt($fileString, 1 + $offset);

        if ($msgNo != 0) {
            $newPointer = $msgNo;
        } else {
            $newPointer = $pointer + 1;
        }

        $fileString = PutLongInt($fileString, $newPointer, 1 + $offset);
        $status = WriteCfgFile($file, $fileString);
        if (!$status) {
            return $status;
        }

        // Return the message pointer.
        return $pointer;
    }


    /**
    * Va-Database.inc.php :: WriteOneliner()
    *
    * This function will create a oneliner message to the specified user. The message
    * can be a maximum of 64 characters.
    *
    * @param integer $toUserNo
    * @param string $message
    * @return mixed
    */
    function WriteOneliner($toUserNo,
                           $message) {
        if (!is_int($toUserNo)) {
            $toUserNo = (integer) $toUserNo;
        }
        if (!is_string($message)) {
            $message = (string)  $message;
        }

        $classNetworks = new vadvNetworks;
        $classUserfileDat = new datUserfile;

        $dbInfo = $this->SetOnelinerInfo();
        $toUser = $classUserfileDat->Get($toUserNo);
        $networks = $classNetworks->GetVnetInfo();

        $file = $GLOBALS['VA_SYSPATHS']['db'] . '/' . $dbInfo['filename'];
        $datFile = $file . '.DAT';
        $binFile = $file . '.BIN';

        $msgBody = "\r\n";

        if (!file_exists($binFile)) {
            $msgpos = 1;
        } else {
            // Get the message starting position.
            $msgpos = filesize($binFile) + 1;
        }

        // Get the message length.
        $msglen = strlen($msgBody);
        if ($msglen > 30000) {
            $msgBody = substr($msgBody, 0, 30000);
            $msglen = strlen($msgBody);
        }

        // Write to the DAT file.
        // Add spacing of 582 bytes of null chars.
        // Create the new blank string.
        $newString = str_pad('', 582, chr(0));

        $newString = PutLongInt($newString, $toUserNo, 1);
        $newString = PutLongInt($newString, $toUserNo, 5);
        $newString = PutString($newString, $toUser['handle'], 9, 40);
        $newString = PutString($newString, $toUser['handle'], 49, 40);
        $newString = PutString($newString, $networks['systemid'], 89, 80);
        $newString = PutString($newString, $networks['systemid'], 169, 80);
        $newString = PutInteger($newString, 1, 249);
        $newString = PutInteger($newString, 1, 251);
        $newString = PutLongInt($newString, lTime(), 253);
        $newString = PutLongInt($newString, $this->WriteTotalMessageCount(), 257);
        $newString = PutLongInt($newString, 0, 261);
        $newString = PutLongInt($newString, 0, 265);
        $newString = PutString($newString, $message, 269, 64);
        $newString = PutInteger($newString, 0, 333);
        $newString = PutString($newString, date("D M d, Y H:i:s O"), 335, 40);
        $newString = PutLongInt($newString, 0, 375);
        $newString = PutLongInt($newString, 0, 379);
        $newString = PutString($newString, '', 383, 12);
        $newString = PutString($newString, '', 395, 64);
        $newString = PutLongInt($newString, 0, 459);
        $newString = PutLongInt($newString, 0, 463);
        $newString = PutLongInt($newString, 0, 467);
        $newString = PutString($newString, $dbInfo['filename'], 471, 8);
        $newString = PutLongInt($newString, $msgpos, 479);
        $newString = PutLongInt($newString, $msglen, 483);
        $newString = PutInteger($newString, 0, 487);
        $newString = PutInteger($newString, 0, 489);
        $newString = PutLongInt($newString, 0, 491);
        $newString = PutString($newString, '', 495, 24);
        $newString = PutString($newString, '', 519, 64);

        $fileString = ReadCfgFile($datFile);
        if ($fileString === FALSE) {
            return FALSE;
        }

        // Error correction...
        $fslen = strlen($fileString);
        $fsnum = $fslen / 582;
        if (is_int($fsnum)) {
            // Append the new data to the original file string.
            $fileString = $fileString . $newString;
        } else {
            $fileString = substr($fileString, 0, intval($fsnum) * 582) . $newString;
        }
        $status = WriteCfgFile($datFile, $fileString);

        // Add the message to the .BIN file.
        // Open the file.
        $fp = fopen($binFile, 'a');
        // Lock the file.
        flock($fp, LOCK_EX);
        // Write the message.
        $status = fwrite($fp, $msgBody);
        // Unlock the file.
        flock($fp, LOCK_UN);
        // Close the file.
        fclose($fp);

        // Return status.
        return $status;
    }


    /**
    * Va-Database.inc.php :: WriteTotalMessageCount()
    *
    * This function will add one to the total message count and return that integer.
    *
    * @return integer
    */
    function WriteTotalMessageCount() {
        $file = $GLOBALS['VA_SYSPATHS']['data'] . '/DBSEQ.DAT';

        if (!file_exists($file)) {
            $fileString = '0';
            $fileString = PutLongInt($fileString, 1, 1);
            $status = WriteCfgFile($file, $fileString);
            if (!$status) {
                return $status;
            }
            return 1;
        }

        // Read the file.
        $fileString = ReadCfgFile($file);

        // Get the total message count.
        $msgcnt = GetLongInt($fileString, 1);

        if ($msgcnt >= 4000000) {
            $msgcnt = 0;
        }

        $fileString = PutLongInt($fileString, $msgcnt + 1, 1);
        $status = WriteCfgFile($file, $fileString);
        if (!$status) {
            return $status;
        }

        // Return the original value.
        return $msgcnt;
    }
}
?>