<?php
/*
 * ----------------------------------------------------------------------------
 * vadv-php_themefunc.inc.php										10/18/2021
 * This script holds all functions for manipulating theme data
 * ----------------------------------------------------------------------------
 * Copyright (C) 2002-2021 Steve Winn. All Rights Reserved.
 * Theme integration by Darrell Fox, Sept-Dec, 2009
 *
 * For licensing information, view LICENSE.TXT.
 * ----------------------------------------------------------------------------
 */
require_once('include_dir.php');
require_once($includeDir . 'vadv-php.inc.php');


$classThemeDat = new datVADVPHPThemes;
$classVADVPHPDat = new datVADVPHP;

$userInfo = $classVADVPHPDat->Get($GLOBALS['USER']['userno']);

// Load user's theme setting and some other theme-related variables --------------------------
// (See /documentation/VADV-PHP Theme.TXT for more information)
$themecount = $classThemeDat->count();

// Check to see if default theme is out of range.
if ($GLOBALS['CONFIG']['defaulttheme'] > $themecount) {
    $GLOBALS['CONFIG']['defaulttheme'] = 1;
}

$themepath = getThemePath();
$themeinfo = array();
for ($i=0; $i<$themecount; $i++) {
	$themeinfo[$i] = $classThemeDat->Get($i+1);
	$themeinfo[$i]['themeID'] = $i+1;
	if (themeDeleted($i+1)) {
		$themeinfo[$i]['cfgMatch'] = 1;
	}else{
		$themeinfo[$i]['cfgMatch'] = checkThemeCfg($i+1);
	}
	$themeinfo[$i]['themeDefault'] = ($GLOBALS['CONFIG']['defaulttheme'] == $i+1) ? 1 : 0;
}
$theme_tracker = $GLOBALS['CONFIG']['themetracker'];
$pagetitle = '';

// $hdr_include is used to include scripts, etc., into the <head> tag of a themed header
// (Done on a page-by-page basis, when you need it)
$hdr_include = '';
// -------------------------------------------------------------------------------------------


/**
* vadv-php_themefunc.inc.php :: getThemeName()
*
* This function returns the theme name based on theme ID passed.
* If there is no matching theme, empty string is returned.
*
* @param int $themeID
* @return string
*/
function getThemeName($themeID) {
	global $classThemeDat;
	$theme = $classThemeDat->Get($themeID);

	if ($theme['themeName'] === '') {
		return '';
	}

	return $theme['themeName'];
}


/**
* vadv-php_themefunc.inc.php :: themeDisabled()
*
* This function returns whether the passed themeID is a
* disabled theme or not
*
* @param int $themeID
* @return boolean
*/
function themeDisabled($themeID) {
	global $classThemeDat;
	$theme = $classThemeDat->Get($themeID);

	if ($theme['themeDisabled'] === 1) {
		return true;
	}
	return false;
}


/**
* vadv-php_themefunc.inc.php :: themeDeleted()
*
* This function returns whether the passed themeID is a
* deleted theme or not
*
* @param int $themeID
* @return boolean
*/
function themeDeleted($themeID) {
	global $classThemeDat;
	$theme = $classThemeDat->Get($themeID);

	if ($theme['themeName'] === '-=DELETED=-') {
		return true;
	}
	return false;
}


/**
* vadv-php_themefunc.inc.php :: getThemePath()
*
* This function checks for the current user's preferred theme
* and, if it's not a disabled theme, returns the path to the
* theme's data files. If the theme is disabled, it returns the
* path of the default theme.
* We 'could' reset their cookie here, but it's not really necessary
* since if a theme is disabled (or deleted), this will make them use
* the default theme, which they will have to change again if they want
* to use a different one, thus resetting the cookie anyway. ;)
*
* @param none
* @return string
*/
function getThemePath() {
	global $rootDir, $userInfo, $themeinfo;

	if ($GLOBALS['USER']['userno'] == '0') {
		$_SESSION['theme'] = isset($_COOKIE['vadvtheme']) ? $_COOKIE['vadvtheme'] : $GLOBALS['CONFIG']['defaulttheme'];
	}else{
		$_SESSION['theme'] = $userInfo['theme'];
	}

	// if theme is disabled and user is under co-sysop level (254) OR theme is deleted, OR theme security level is different than user, set default theme for user
	if ((themeDisabled($_SESSION['theme']) && $GLOBALS['USER']['securitylevel'] < 254) ||
		themeDeleted($_SESSION['theme']) || getThemeName($_SESSION['theme']) == '' ||
		(($themeinfo !== null) && ($themeinfo[$_SESSION['theme']-1]['themeSecLevel'] > $GLOBALS['USER']['securitylevel']))) {
		$_SESSION['theme'] = $GLOBALS['CONFIG']['defaulttheme'];
	}

	return $rootDir . "themes/" . getThemeName($_SESSION['theme']);
}



/**
* vadv-php_themefunc.inc.php :: VADV_LoginError()
*
* This function is a replacement for the VA_LoginError() found
* in va-functions.inc.php so we can extend the theme implementation
* without touching the original va-functions.inc.php file.
* (Yes, that means either this function can go there or that function
* could be deleted, but we're not at this time) ;)
*
* @param none
* @return none
*/
function VADV_LoginError() {
	global $rootDir, $includeDir, $themepath, $theme_tracker, $startTime, $VERSION;

	// Output header, page & footer
	$pagetitle = 'Log In Error';
	include($themepath . '/templates/header.php');
	include($themepath . '/templates/loginerror.php');
	include($themepath . '/templates/footer.php');
}



/**
* vadv-php_themefunc.inc.php :: IsOdd()
*
* (One of Darrell's handy little func's - put here so as not to
* mess with original VADV-PHP function include files any more
* than necessary) ;)
*
* Used to judge odd or even itenerations in a loop for swapping row
* classes, for instance
*
* @param none
* @return int  (0 if 'even' number, 1 if 'odd' number)
*
*/
function IsOdd($number) {
   return $number & 1; // 0 = even, 1 = odd
}



/**
* vadv-php_themefunc.inc.php :: isAlphaNumeric()
*
* (One of Darrell's handy little func's - put here so as not to
* mess with original VADV-PHP function include files any more
* than necessary) ;)
*
* Used to validate whether a given string consists of only alpha-
* numeric characters (a-z, A-Z, 0-9)
*
* @param string $str
* @param string $except (exception chars to allow in addition to the above)
* @return boolean
*
*/
function isAlphaNumeric($str, $except=''){
	return (strspn($str, "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789".$except) == strlen($str));
}



/**
* vadv-php_themefunc.inc.php :: ThemeFixHeartColors()
*
* This function replaces the original FixHeartColors() located in
* va-functions.inc.php so the original include file didn't have to
* be modified.
*
* @param string $string
* @param $defaultcolor (CSS hex color code/name)
* @param $defaultfont (pixel size)
* @return string
*/
function ThemeFixHeartColors($string,
						$defaultcolor = '',
						$defaultfont = '') {
	if (!is_string($string)) {
		$string = (string) $string;
	}
	if (!is_string($defaultcolor)) {
		$defaultcolor = (string) $defaultcolor;
	}
	if (!is_string($defaultfont)) {
		$defaultfont = (string) $defaultfont;
	}

	// Check to see if the variable was passed, if not then return the string.
	if (trim($string) == '') {
		return $string;
	}

	// Temporary flag for span tagss.
	$spanFlag = 0;

	// Search for hearts.
	for ($position = 0; is_int($position);) {
		$position = strpos($string, chr(3), 0);

		// If found a heart...
		if (is_int($position)) {
			$code = substr($string, $position + 1, 1);
			switch (strtoupper($code)) {
				case '0':
					$color = '#ADAAAD';
					break;
				case '1':
					$color = '#52FFFF';
					break;
				case '2':
					$color = '#FFFF52';
					break;
				case '3':
					$color = '#AD00AD';
					break;
				case '4':
					$color = '#FFFFFF';
					break;
				case '5':
					$color = '#52FF52';
					break;
				case '6':
					$color = '#FF5552';
					break;
				case '7':
					$color = '#5255FF';
					break;
				case '8':
					$color = '#AD5500';
					break;
				case '9':
					$color = '#00AAAD';
					break;
				case 'A':
					$color = '#00AA00';
					break;
				case 'B':
					$color = '#FF55FF';
					break;
				case 'C':
					$color = '#FFFFFF';
					break;
				case 'D':
					$color = '#FFFF52';
					break;
				case 'E':
					$color = '#52FFFF';
					break;
				case 'F':
					$color = '#ADAAAD';
					break;
				case 'G':
					$color = '#5255FF';
					break;
				case 'H':
					$color = '#FF5552';
					break;
				case 'I':
					$color = '#5255FF';
					break;
				case 'J':
					$color = '#FFFF52';
					break;
				case 'K':
					$color = '#52FFFF';
					break;
				case 'L':
					$color = '#ADAAAD';
					break;
				case 'M':
					$color = '#FFFFFF';
					break;
				case 'N':
					$color = '#ADAAAD';
					break;
				case 'O':
					$color = '#ADAAAD';
					break;
				case 'P':
					$color = '#AD0000';
					break;
				case 'Q':
					$color = '#0000AD';
					break;
				case 'R':
					$color = '#FFFFFF';
					break;
				case 'S':
					$color = '#FF5552';
					break;
				case 'T':
					$color = '#5255FF';
					break;
				case 'U':
					$color = '#FFFFFF';
					break;
				case 'V':
					$color = '#ADAAAD';
					break;
				case 'W':
					$color = '#ADAAAD';
					break;
				case 'X':
					$color = '#ADAAAD';
					break;
				case 'Y':
					$color = '#FFFFFF';
					break;
				case 'Z':
					$color = '#FFFFFF';
					break;
				case '[':
					$color = '#525552';
					break;
				default:
					$color = $defaultcolor;
					break;
			}

			// If a default color is specified, make all text that color
			if ($defaultcolor != '') { $color = $defaultcolor; }

			// If a font-size is specified, add that also
			$fontsize = '';
			if ($defaultfont != '') { $fontsize = 'font-size: ' . $defaultfont . 'px;'; }

			if ($color != '') {
				// Insert the new information into the string.
				$a = substr($string, 0, $position);
				$b = substr($string, $position + 2);
				$string = $a . '<span style="' . $fontsize . 'color: ' . $color . '">' . $b;

				// Change span flag.
				$spanFlag = 1;

				// Check for next heart code for closing tag.
				$position = strpos($string, chr(3), 0);
				if (is_int($position)) {
					$a = substr($string, 0, $position);
					$b = substr($string, $position);
					$string = $a . '</span>' . $b;
				}
			} else {
				$a = substr($string, 0, $position);
				$b = substr($string, $position + 1);
				$string = $a . $b;

				$spanFlag = 0;
			}
		}
	}

	if ($spanFlag == 1) {
		$string = $string . '</span>';
	}

	// Return the new string.
	return $string;
}



/**
* vadv-php_themefunc.inc.php :: showHelp()
*
* This function places a Help icon in the spot called from with link
* for both script and non-script browsers to open the help window
*
* @param string $helpSection (First part of help file name)
* @param string $helpID (name of the #anchor in help file)
* @return none (direct output)
*/
function showHelp($helpSection, $helpID) {
	global $rootDir, $themepath;

	$helpPath = $themepath . '/help/' . $helpSection . '_help.php';

	print '<script language="JavaScript" type="text/javascript">' . "\n";
	print 'document.write(\'<a href="Javascript:HelpWindow(\\\'' . $helpPath . '\\\',\\\'' . $helpID . '\\\')"><img border="0" src="' . $themepath . '/img/help.gif" alt="Help" width="15" title="Help" height="15" align="top"></a>\');' . "\n";
	print '</script>' . "\n";
	print '<noscript>' . "\n";
	print '<a href="' . $helpPath.'#' . $helpID . '" target="helpwindow"><img border="0" src="' . $themepath . '/img/help.gif" alt="Help" title="Help" width="15" height="15" align="top"></a>' . "\n";
	print '</noscript>' . "\n";
}



/**
* vadv-php_themefunc.inc.php :: genTZOptions()
*
* This function generates the <select> option list for all
* time zones needed
*
* @param none
* @return none (direct output)
*/
function genTZOptions() {

	print '<option value="0">Reset to BBS Time</option>' . "\n";
	print '<option value="-1200" ' . (($GLOBALS['USER_SETTINGS']['timezone'] == -1200) ? "selected=\"selected\"" : "") . '>GMT -12 Hours</option>' . "\n";
	print '<option value="-1100" ' . (($GLOBALS['USER_SETTINGS']['timezone'] == -1100) ? "selected=\"selected\"" : "") . '>GMT -11 Hours</option>' . "\n";
	print '<option value="-1000" ' . (($GLOBALS['USER_SETTINGS']['timezone'] == -1000) ? "selected=\"selected\"" : "") . '>GMT -10 Hours</option>' . "\n";
	print '<option value="-950" ' . (($GLOBALS['USER_SETTINGS']['timezone'] == -950) ? "selected=\"selected\"" : "" ) . '>GMT -9:30 Hours</option>' . "\n";
	print '<option value="-900" ' . (($GLOBALS['USER_SETTINGS']['timezone'] == -900) ? "selected=\"selected\"" : "" ) . '>GMT -9 Hours</option>' . "\n";
	print '<option value="-800" ' . (($GLOBALS['USER_SETTINGS']['timezone'] == -800) ? "selected=\"selected\"" : "" ) . '>GMT -8 Hours</option>' . "\n";
	print '<option value="-700" ' . (($GLOBALS['USER_SETTINGS']['timezone'] == -700) ? "selected=\"selected\"" : "" ) . '>GMT -7 Hours</option>' . "\n";
	print '<option value="-600" ' . (($GLOBALS['USER_SETTINGS']['timezone'] == -600) ? "selected=\"selected\"" : "" ) . '>GMT -6 Hours</option>' . "\n";
	print '<option value="-500" ' . (($GLOBALS['USER_SETTINGS']['timezone'] == -500) ? "selected=\"selected\"" : "" ) . '>GMT -5 Hours</option>' . "\n";
	print '<option value="-400" ' . (($GLOBALS['USER_SETTINGS']['timezone'] == -400) ? "selected=\"selected\"" : "" ) . '>GMT -4 Hours</option>' . "\n";
	print '<option value="-350" ' . (($GLOBALS['USER_SETTINGS']['timezone'] == -350) ? "selected=\"selected\"" : "" ) . '>GMT -3:30 Hours</option>' . "\n";
	print '<option value="-300" ' . (($GLOBALS['USER_SETTINGS']['timezone'] == -300) ? "selected=\"selected\"" : "" ) . '>GMT -3 Hours</option>' . "\n";
	print '<option value="-200" ' . (($GLOBALS['USER_SETTINGS']['timezone'] == -200) ? "selected=\"selected\"" : "" ) . '>GMT -2 Hours</option>' . "\n";
	print '<option value="-100" ' . (($GLOBALS['USER_SETTINGS']['timezone'] == -100) ? "selected=\"selected\"" : "" ) . '>GMT -1 Hour</option>' . "\n";
	print '<option value="1" ' . (($GLOBALS['USER_SETTINGS']['timezone'] == 0) ? "selected=\"selected\"" : "" ) . '>GMT</option>' . "\n";
	print '<option value="100" ' . (($GLOBALS['USER_SETTINGS']['timezone'] == 100) ? "selected=\"selected\"" : "" ) . '>GMT +1 Hour</option>' . "\n";
	print '<option value="200" ' . (($GLOBALS['USER_SETTINGS']['timezone'] == 200) ? "selected=\"selected\"" : "" ) . '>GMT +2 Hours</option>' . "\n";
	print '<option value="300" ' . (($GLOBALS['USER_SETTINGS']['timezone'] == 300) ? "selected=\"selected\"" : "" ) . '>GMT +3 Hours</option>' . "\n";
	print '<option value="350" ' . (($GLOBALS['USER_SETTINGS']['timezone'] == 350) ? "selected=\"selected\"" : "" ) . '>GMT +3:30 Hours</option>' . "\n";
	print '<option value="400" ' . (($GLOBALS['USER_SETTINGS']['timezone'] == 400) ? "selected=\"selected\"" : "" ) . '>GMT +4 Hours</option>' . "\n";
	print '<option value="450" ' . (($GLOBALS['USER_SETTINGS']['timezone'] == 450) ? "selected=\"selected\"" : "" ) . '>GMT +4:30 Hours</option>' . "\n";
	print '<option value="500" ' . (($GLOBALS['USER_SETTINGS']['timezone'] == 500) ? "selected=\"selected\"" : "" ) . '>GMT +5 Hours</option>' . "\n";
	print '<option value="550" ' . (($GLOBALS['USER_SETTINGS']['timezone'] == 550) ? "selected=\"selected\"" : "" ) . '>GMT +5:30 Hours</option>' . "\n";
	print '<option value="575" ' . (($GLOBALS['USER_SETTINGS']['timezone'] == 575) ? "selected=\"selected\"" : "" ) . '>GMT +5:45 Hours</option>' . "\n";
	print '<option value="600" ' . (($GLOBALS['USER_SETTINGS']['timezone'] == 600) ? "selected=\"selected\"" : "" ) . '>GMT +6 Hours</option>' . "\n";
	print '<option value="650" ' . (($GLOBALS['USER_SETTINGS']['timezone'] == 650) ? "selected=\"selected\"" : "" ) . '>GMT +6:30 Hours</option>' . "\n";
	print '<option value="700" ' . (($GLOBALS['USER_SETTINGS']['timezone'] == 700) ? "selected=\"selected\"" : "" ) . '>GMT +7 Hours</option>' . "\n";
	print '<option value="800" ' . (($GLOBALS['USER_SETTINGS']['timezone'] == 800) ? "selected=\"selected\"" : "" ) . '>GMT +8 Hours</option>' . "\n";
	print '<option value="875" ' . (($GLOBALS['USER_SETTINGS']['timezone'] == 875) ? "selected=\"selected\"" : "" ) . '>GMT +8:45 Hours</option>' . "\n";
	print '<option value="900" ' . (($GLOBALS['USER_SETTINGS']['timezone'] == 900) ? "selected=\"selected\"" : "" ) . '>GMT +9 Hours</option>' . "\n";
	print '<option value="950" ' . (($GLOBALS['USER_SETTINGS']['timezone'] == 950) ? "selected=\"selected\"" : "" ) . '>GMT +9:30 Hours</option>' . "\n";
	print '<option value="1000" ' . (($GLOBALS['USER_SETTINGS']['timezone'] == 1000) ? "selected=\"selected\"" : "" ) . '>GMT +10 Hours</option>' . "\n";
	print '<option value="1050" ' . (($GLOBALS['USER_SETTINGS']['timezone'] == 1050) ? "selected=\"selected\"" : "" ) . '>GMT +10:30 Hours</option>' . "\n";
	print '<option value="1100" ' . (($GLOBALS['USER_SETTINGS']['timezone'] == 1100) ? "selected=\"selected\"" : "" ) . '>GMT +11 Hours</option>' . "\n";
	print '<option value="1150" ' . (($GLOBALS['USER_SETTINGS']['timezone'] == 1150) ? "selected=\"selected\"" : "" ) . '>GMT +11:30 Hours</option>' . "\n";
	print '<option value="1200" ' . (($GLOBALS['USER_SETTINGS']['timezone'] == 1200) ? "selected=\"selected\"" : "" ) . '>GMT +12 Hours</option>' . "\n";
	print '<option value="1275" ' . (($GLOBALS['USER_SETTINGS']['timezone'] == 1275) ? "selected=\"selected\"" : "" ) . '>GMT +12:45 Hours</option>' . "\n";
	print '<option value="1300" ' . (($GLOBALS['USER_SETTINGS']['timezone'] == 1300) ? "selected=\"selected\"" : "" ) . '>GMT +13 Hours</option>' . "\n";
	print '<option value="1400" ' . (($GLOBALS['USER_SETTINGS']['timezone'] == 1400) ? "selected=\"selected\"" : "" ) . '>GMT +14 Hours</option>' . "\n";

}



/**
* vadv-php_themefunc.inc.php :: genStateOptions()
*
* This function generates the <select> option list for all
* states needed
*
* This should be modified later to include other territory abbreviations,
* provinces, etc that we can use (up to 3 chars)
*
* (First one in array should be "" for select boxes where we pass no default
* state abbrev to the function so it shows blank)
*
* @param string (2-3 char abbrev for state to match for default "selected" option)
* @return none (direct output)
*/
function genStateOptions($st)
{
	$states = array("","AL","AK","AZ","AR","CA","CO","CT","DE","DC","FL","GA","HI","ID","IL","IN","IA","KS","KY","LA","ME","MD","MA","MI","MN","MS","MO","MT","NE","NV","NH","NJ","NM","NY","NC","ND","OH","OK","OR","PA","RI","SC","SD","TN","TX","UT","VT","VA","WA","WV","WI","WY");

	if ($st == '')
	{
		print '<option value="" selected="selected">  </option>';
	}else{
		print '<option value="">  </option>';
	}
	print '<optgroup label="-USA-">';
	foreach($states as $value)
	{
		if ($value == $st)
		{
			print '<option value="'.$value.'" selected="selected">'.$value.'</option>';
		}else{
			print '<option value="'.$value.'">'.$value.'</option>';
		}
	}
	print '</optgroup>';

	$provinces = array("AB","BC","MB","NB","NL","NT","NS","NU","ON","PE","QC","SK","YT");

	print '<optgroup label="-CAN-">';
	foreach($provinces as $value)
	{
		if ($value == $st)
		{
			print '<option value="'.$value.'" selected="selected">'.$value.'</option>';
		}else{
			print '<option value="'.$value.'">'.$value.'</option>';
		}
	}
	print '</optgroup>';
}



/**
* vadv-php_themefunc.inc.php :: convertTZ()
*
* This function converts stored timezone integer into
* a string for display (e.g., -500 to '-5', or -950 to '-9:30')
*
* @param integer $inTZ
* @return string
*/
function convertTZ($inTZ) {

	if (!is_int($inTZ)) {
		$inTZ = (int) $inTZ;
	}

	$zones = array(-1200 => '-12', -1100 => '-11', -1000 => '-10', -950 => '-9:30', -900 => '-9', -800 => '-8', -700 => '-7', -600 => '-6', -500 => '-5', -400 => '-4', -350 => '-3:30',
		-300 => '-3', -200 => '-2', -100 => '-1', 1 => '', 100 => '+1', 200 => '+2', 300 => '-3', 350 => '-3:30', 400 => '-4', 450 => '-4:30', 500 => '-5', 550 => '-5:30', 575 => '-5:45',
		600 => '-6', 650 => '-6:30', 700 => '-7', 800 => '-8', 875 => '-8:45', 900 => '-9', 950 => '-9:30', 1000 => '+10', 1050 => '+10:30', 1100 => '+11', 1150 => '+11:30',
		1200 => '+12', 1275 => '+12:45', 1300 => '+13', 1400 => '+14');

	foreach($zones as $key => $value) {
		if ($inTZ === $key) {
			return $value;
		}
	}

	return '';

}



/**
* vadv-php_themefunc.inc.php :: genMailFwdOptions()
*
* This function generates the <select> option list for all
* user Mail Forward choices, local and network (if any)
*
* @param none
* @return none (direct output)
*/
function genMailFwdOptions() {

	$classNetworks = new vadvNetworks;
	$VA_NETWORKS = $classNetworks->GetNetworkInfo();

	print '<option ' . ((($GLOBALS['USER']['mailfwdflag'] == 1) && ($GLOBALS['USER']['mailfwdnetno'] == 1) && ($GLOBALS['USER']['mailfwdnetnode'] == $VA_NEWORKS[1]['settings']['systemid'])) ? "selected=\"selected\"" : "") . ' value="0">Local</option>' . "\n";

	$cnt = 0;
	$net = array();
	foreach($VA_NETWORKS as $netInfo) {
		if ((isset($netInfo['nettype'])) &&
			($netInfo['nettype'] != 2) &&   // If not a VLive network
			($netInfo['nettype'] != 5)) {   // If not a QWK network

			if ((isset($netInfo['settings']['systemid'])) && ($netInfo['settings']['systemid'] != '0')) {
				if (($GLOBALS['USER']['securitylevel'] >= 255) ||
					($netInfo['accflag'] == '') ||  // If a access flag is set..
					(strpos($GLOBALS['USER']['flags'], $netInfo['accflag']) !== FALSE)) {   // If the user has the flag

						switch ($netInfo['nettype']) {
							case 1:
								$type = 'VirtualNET';
								break;
							case 3:
								$type = 'Internet';
								break;
							case 4:
								$type = 'FIDOnet';
								break;
							case 9:
								$type = 'WWIVnet';
								break;
							default:
								$type = 'Unknown';
								break;
						}

						$net[$cnt]['netno'] = $netInfo['netno'];
						$net[$cnt]['desc'] = $netInfo['desc'];
						$net[$cnt]['sortdesc'] = strtolower($netInfo['desc']);
						$net[$cnt]['type'] = $type;
						$cnt++;
					}
				}
			}
		}

		// Organize the array alphabetically by description.
		$sortarr = array();
		foreach ($net as $row) {
			$sortarr[] = $row['sortdesc'];
		}
		array_multisort($sortarr, $net);

		// Display the listing.
		for ($i = 0; $i < count($net); $i++) {
			print '<option ' . ((($GLOBALS['USER']['mailfwdflag'] == 1) && ($GLOBALS['USER']['mailfwdnetno'] == $net[$i]['netno'])) ? "selected=\"selected\"" : "") . 'value="' . $net[$i]['netno'] . '">' . $net[$i]['desc'] . ' (' . $net[$i]['type'] . ")</option>\n";
		}
}



/**
* vadv-php_themefunc.inc.php :: genThemeOptions()
*
* This function generates the <select> option list for all
* installed themes for the user to choose (disabled themes
* are not listed)
*
* @param none
* @return none (direct output)
*/
function getThemeOptions() {

	global $classThemeDat;

	$themes = $classThemeDat->count();

	for ($i=0; $i<$themes; $i++) {
		print '<option value="' . ($i+1) . '" ' . (($GLOBALS['USER_SETTINGS']['theme'] == ($i+1)) ? "selected=\"selected\"" : "") . '>' . getThemeName($i+1) . '</option>' . "\n";
	}
}



/**
* vadv-php_themefunc.inc.php :: checkThemeCfg()
*
* This function checks an installed theme's version number,
* author's name, author's email, author's website and date
* against the theme's vadvtheme.cfg to see if someone has
* unpacked a newer version of the theme and the info in
* vadv-php_themes.dat needs to be updated
*
* @param int $themeID
* @return boolean
*/
function checkThemeCfg($themeID) {

	global $classThemeDat, $rootDir;

	if (!is_int($themeID)) {
		$themeID = (int)$themeID;
	}

	$themeData = $classThemeDat->Get($themeID);

    $name = $themeData['themeName'];
	$ver = $themeData['themeVer'];
	$date = $themeData['themeDate'];
	$author = $themeData['themeAuthor'];
	$email = $themeData['themeEmail'];
	$url = $themeData['themeSite'];

	$themeDir = $rootDir . 'themes/' . $themeData['themeName'];

	$themeFile = @fopen($themeDir . '/vadvtheme.cfg', "rt");

	while ($line = fgets($themeFile)) {
		if (strstr($line, "name=")) {
			$ptr = trim(strtok($line, "="));
			$name = trim(strtok("="));
		}
		if (strstr($line, "ver=")) {
			$ptr = trim(strtok($line, "="));
			$ver = trim(strtok("="));
		}
		if (strstr($line, "date=")) {
			$ptr = trim(strtok($line, "="));
			$ptr = trim(strtok("="));
			$date = strtotime($ptr);
		}
		if (strstr($line, "author=")) {
			$ptr = trim(strtok($line, "="));
			$author = trim(strtok("="));
		}
		if (strstr($line, "authoremail=")) {
			$ptr = trim(strtok($line, "="));
			$email = trim(strtok("="));
		}
		if (strstr($line, "authorsite=")) {
			$ptr = trim(strtok($line, "="));
			$url = trim(strtok("="));
		}
	}

	fclose($themeFile);

	/*print "VER: " . $ver . " - " . $themeData['themeVer'] . "<br />";
	print "DATE: " .  date("m/d/Y", $date) . " - " .  date("m/d/Y", $themeData['themeDate']) . "<br />";
	print "AUTHOR: " . $author . " - " . $themeData['themeAuthor'] . "<br />";
	print "EMAIL: " . $email . " - " . $themeData['themeEmail'] . "<br />";
	print "URL: " . $url . " - " . $themeData['themeSite'] . "<br />";
	exit;*/

	if ($name != $themeData['themeName'] || $ver != $themeData['themeVer'] || date("m/d/Y", $date) != date("m/d/Y", $themeData['themeDate']) ||
		$author != $themeData['themeAuthor'] || $email != $themeData['themeEmail'] ||
		$url != $themeData['themeSite']) {
		return false;
	}else{
		return true;
	}
}



/**
* vadv-php_themefunc.inc.php :: createNewTheme()
*
* This function copies a theme into a new theme directory and
* writes the vadvtheme.cfg ready for installation
*
* Passed array must contain:
*		(string)['themeName'] - Name of new theme (16 char max, alphanumeric)
*		(string)['themeAuthor'] - Name of new theme author
*		(string)['themeEmail'] - Email of new theme author
*		(string)['themeSite'] - Site (if any) of new theme author
*
* (Note: Theme version will automatically be set to 1.0 and the theme date will
* be set to the date of creation (when this function runs).)
*
*
* Also note that this is written on and for a Windows system, modification will
* probably be needed to work on 'nix systems, etc..
*
*
* @param int $cloneID (id of the installed theme to copy
* @param array $newThemeInfo (array containing config info)
* @return boolean
*/
function createNewTheme($cloneID, $newThemeInfo) {
	global $rootDir, $classThemeDat;

	if (!is_int($cloneID)) {
		$cloneID = (int) $cloneID;
	}

	if (!is_array($newThemeInfo)) {
		// Why doesn't this exit the functions? It's in every class's Put() function also, but doesn't exit,
		// just returns an error message and continues with execution on an invalid variable that's not an array...
		// @TODO: end functions and properly show error messages... ;) -D-
		trigger_error(VA_Error(1));
	}

	if (is_dir($rootDir . 'themes/' . $newThemeInfo['themeName'])) {
		return false;
	}

	// Everything looks good, let's create the new theme!
	$themeData = $classThemeDat->Get($cloneID);

	// Create directories & copy requested theme files into new directory
	$sourceDir = $rootDir . 'themes/' . $themeData['themeName'] . '/';
	$destDir = $rootDir . 'themes/' . $newThemeInfo['themeName'] . '/';
	$newThemeInfo['themeVer'] = '1.0';
	$result = smartCopy($sourceDir, $destDir);

	// If that went well, let's edit the vadvtheme.cfg file so the new theme
	// can be installed
	if ($result) {
		writeThemeCfg($newThemeInfo, $destDir);
	}

	return $result;
}




/**
* vadv-php_themefunc.inc.php :: verifyThemeCfg()
*
* This function examines the vadvtheme.cfg located
* in a theme directory and checks to see if it's a
* valid theme config
*
* @param string (theme/directory name)
* @return boolean
*/
function verifyThemeCfg($dirName) {
	global $rootDir;

	$cfile = file($rootDir . 'themes/' . $dirName . '/vadvtheme.cfg', FILE_IGNORE_NEW_LINES | FILE_SKIP_EMPTY_LINES);

	$checks = 0;
	foreach($cfile as $line) {
		if (!strncmp("name=", $line, 5)) {
			$ptr = strtok($line, "=");
			$name = trim(strtok("="));
			// Make sure name in config matches the directory name
			if ($name == $dirName) {
				$checks++;
			}
			continue;
		}
		if (!strncmp("ver=", $line, 4)) {
			$ptr = strtok($line, "=");
			$ver = trim(strtok("="));
			// Basic check to make sure version isn't blank (if it becomes a problem, we can add more checks if we want)
			if ($ver != '') {
				$checks++;
			}
			continue;
		}
		if (!strncmp("date=", $line, 5)) {
			$ptr = strtok($line, "=");
			$date = trim(strtok("="));
			// Basic check to see if we have a valid date format by looking for the proper placements of /'s and length (we can always add more checks if we want)
			if (!substr_compare($date, "/", 2, 1) && !substr_compare($date, "/", 5, 1) && strlen($date) == 10) {
				$checks++;
			}
			continue;
		}
		if (!strncmp("author=", $line, 7)) {
			$checks++;
			continue;
		}
		if (!strncmp("authoremail=", $line, 12)) {
			$checks++;
			continue;
		}
		if (!strncmp("authorsite=", $line, 11)) {
			$checks++;
			continue;
		}

	}

	if ($checks == 6) {
		return true;
	}

	return false;
}




/**
* vadv-php_themefunc.inc.php :: searchForThemes()
*
* This function retrieves a list of theme directories in
* the /themes folder and checks to see if there are any
* new theme packages to install
*
* @param none
* @return array (containing all new directory names)
*/
function searchForThemes() {
	global $classThemeDat, $rootDir, $themeinfo, $themecount;

	$newThemes = array();

	// Get current installed theme list
	$currentThemes = array();
	for ($i=0; $i<$themecount; $i++) {
		$currentThemes[$i] = $themeinfo[$i]['themeName'];
	}

	// Get theme directory listing
	$contents = scandir($rootDir . 'themes');

	// Search for new theme directories
	for ($i=0; $i<count($contents); $i++) {
		if ($contents[$i] == '.' || $contents[$i] == '..' || $contents[$i] == 'vadv-php_themes.dat' || in_array($contents[$i], $currentThemes)) {
			continue;
		}

		// Check for and verify new directory's vadvtheme.cfg
		if (file_exists($rootDir . 'themes/' . $contents[$i] . '/vadvtheme.cfg')) {
			if (verifyThemeCfg($contents[$i])) {
				$newThemes[] = $contents[$i];
			}
		}
	}

	return $newThemes;

}




/**
* vadv-php_themefunc.inc.php :: getThemeCfg()
*
* This function retrieves a themes information from the
* vadvtheme.cfg file in the theme's root directory for
* use in updating or installing a theme.
*
* @param string $themeName
* @return array
*/
function getThemeCfg($themeName) {

	global $rootDir;
	$themeCfg = array();

	if (!is_string($themeName)) {
		$themeName = (string)$themeName;
	}

	$themeDir = $rootDir . 'themes/' . $themeName;

	if (!is_dir($themeDir)) {
		return false;
	}

	$themeFile = @fopen($themeDir . '/vadvtheme.cfg', "rt");

	while ($line = fgets($themeFile)) {
		if (strstr($line, "name=")) {
			$ptr = trim(strtok($line, "="));
			$themeCfg['themeName'] = trim(strtok("="));
		}
		if (strstr($line, "ver=")) {
			$ptr = trim(strtok($line, "="));
			$themeCfg['themeVer'] = trim(strtok("="));
		}
		if (strstr($line, "date=")) {
			$ptr = trim(strtok($line, "="));
			$ptr = trim(strtok("="));
			$themeCfg['themeDate'] = strtotime($ptr);
		}
		if (strstr($line, "author=")) {
			$ptr = trim(strtok($line, "="));
			$themeCfg['themeAuthor'] = trim(strtok("="));
		}
		if (strstr($line, "authoremail=")) {
			$ptr = trim(strtok($line, "="));
			$themeCfg['themeEmail'] = trim(strtok("="));
		}
		if (strstr($line, "authorsite=")) {
			$ptr = trim(strtok($line, "="));
			$themeCfg['themeSite'] = trim(strtok("="));
		}
	}

	fclose($themeFile);

	return $themeCfg;
}



/**
* vadv-php_themefunc.inc.php :: writeThemeCfg()
*
* This function updates the vadvtheme.cfg in passed theme's directory
*
* Passed array must contain:
*		(string)['themeName'] - Name of new theme (16 char max, alphanumeric)
*		(strint)['themeVer'] - New version number
*	--> (Theme Date Will Be Automatically Created With Current Date When Function Runs)
*		(string)['themeAuthor'] - Name of new theme author
*		(string)['themeEmail'] - Email of new theme author
*		(string)['themeSite'] - Site (if any) of new theme author
*		(string)['themeSite'] - Site (if any) of new theme author
*
* @param array $newThemeInfo (array containing config info)
* @param string $destDir (destination directory containing vadvtheme.cfg for this theme)
* @return boolean
*/
function writeThemeCfg($newThemeInfo, $destDir) {

	if (!is_array($newThemeInfo)) {
		// Why doesn't this exit the functions? It's in every class's Put() function also, but doesn't exit,
		// just returns an error message and continues with execution on an invalid variable that's not an array...
		// @TODO: end functions and properly show error messages... ;) -D-
		trigger_error(VA_Error(1));
	}

	$cfgfile = @fopen($destDir . 'vadvtheme.cfg', "wt");

		fwrite($cfgfile, "#--------------------------------------------------------#\n");
		fwrite($cfgfile, "#  VADV-PHP Theme Configuration File                     #\n");
		fwrite($cfgfile, "#  For use with VADV-PHP 2.0 and higher                  #\n");
		fwrite($cfgfile, "#  VADV-PHP Copyright �2009 by Steve Winn                #\n");
		fwrite($cfgfile, "#  http://www.vadvphp.com/                               #\n");
		fwrite($cfgfile, "#--------------------------------------------------------#\n\n");
		fwrite($cfgfile, "name=" . $newThemeInfo['themeName'] . "\n");
		fwrite($cfgfile, "ver=" . $newThemeInfo['themeVer'] . "\n");
		fwrite($cfgfile, "date=" . date("m", time()) . '/' . date("d", time()) . '/' . date("Y", time()) . "\n");
		fwrite($cfgfile, "author=" . $newThemeInfo['themeAuthor'] . "\n");
		fwrite($cfgfile, "authoremail=" . $newThemeInfo['themeEmail'] . "\n");
		fwrite($cfgfile, "authorsite=" . $newThemeInfo['themeSite'] . "\n");

		fclose($cfgfile);
}



/**
* vadv-php_themefunc.inc.php :: smartCopy()
*
* (Taken directly from member contributions at www.php.net, specifically:
* http://us2.php.net/copy  -->  "Sina Salek"  -->  modified by  -->  "info ]t[ intalo [.]de"
*
* Copy file or folder from source to destination, it can do recursive copy as well and is very smart
* It recursively creates the dest file or directory path if there weren't exists
*
* Situtaions :
* - Src:/home/test/file.txt ,Dst:/home/test/b ,Result:/home/test/b -> If source was file copy file.txt name with b as name to destination
* - Src:/home/test/file.txt ,Dst:/home/test/b/ ,Result:/home/test/b/file.txt -> If source was file Creates b directory if does not exsits and copy file.txt into it
* - Src:/home/test ,Dst:/home/ ,Result:/home/test/** -> If source was directory copy test directory and all of its content into dest
* - Src:/home/test/ ,Dst:/home/ ,Result:/home/**-> if source was direcotry copy its content to dest
* - Src:/home/test ,Dst:/home/test2 ,Result:/home/test2/** -> if source was directoy copy it and its content to dest with test2 as name
* - Src:/home/test/ ,Dst:/home/test2 ,Result:->/home/test2/** if source was directoy copy it and its content to dest with test2 as name
* @todo
*     - Should have rollback technique so it can undo the copy when it wasn't successful
*  - Auto destination technique should be possible to turn off
*  - Supporting callback function
*  - May prevent some issues on shared enviroments : http://us3.php.net/umask
* @param $source //file or folder
* @param $dest ///file or folder
* @param $options //folderPermission,filePermission
* @return boolean
*/
function smartCopy($source, $dest, $folderPermission='0755',$filePermission='0644'){
// source=file & dest=dir => copy file from source-dir to dest-dir
// source=file & dest=file / not there yet => copy file from source-dir to dest and overwrite a file there, if present

// source=dir & dest=dir => copy all content from source to dir
// source=dir & dest not there yet => copy all content from source to a, yet to be created, dest-dir
	$result=false;

	if (is_file($source)) { // $source is file
		if(is_dir($dest)) { // $dest is folder
			if ($dest[strlen($dest)-1]!='/') // add '/' if necessary
				$__dest=$dest."/";
			$__dest .= basename($source);
			}
		else { // $dest is (new) filename
			$__dest=$dest;
			}
		$result=copy($source, $__dest);
		chmod($__dest,$filePermission);
		}
	elseif(is_dir($source)) { // $source is dir
		if(!is_dir($dest)) { // dest-dir not there yet, create it
			@mkdir($dest,$folderPermission);
			chmod($dest,$folderPermission);
			}
		if ($source[strlen($source)-1]!='/') // add '/' if necessary
			$source=$source."/";
		if ($dest[strlen($dest)-1]!='/') // add '/' if necessary
			$dest=$dest."/";

		// find all elements in $source
		$return = true; // in case this dir is empty it would otherwise return false
		$dirHandle=opendir($source);
		while($file=readdir($dirHandle)) { // note that $file can also be a folder
			if($file!="." && $file!="..") { // filter starting elements and pass the rest to this function again
//				print "$source$file ||| $dest$file<br />\n";
				$result=smartCopy($source.$file, $dest.$file, $folderPermission, $filePermission);
				}
			}
		closedir($dirHandle);
		}
	else {
		$result=false;
		}
	return $result;
}




/**
* vadv-php_themefunc.inc.php :: smartCopy()
*
* Recursively delete a directory
*
* @param string $dir Directory name
* @param boolean $deleteRootToo Delete specified top-level directory as well
*/
function unlinkRecursive($dir, $deleteRootToo)
{
    if(!$dh = opendir($dir))
    {
        return;
    }
    while (($obj = readdir($dh)) !== false)
    {
        if($obj == '.' || $obj == '..')
        {
            continue;
        }

        if (!@unlink($dir . '/' . $obj))
        {
            unlinkRecursive($dir.'/'.$obj, true);
        }
    }

    closedir($dh);

    if ($deleteRootToo)
    {
        rmdir($dir);
    }

    return;
}




/*$ARCHIVE = new zip;

$ARCHIVE->makeZip('./','./toto.zip'); // make an ZIP archive
var_export($ARCHIVE->infosZip('./toto.zip'), false); // get infos of this ZIP archive (without files content)
var_export($ARCHIVE->infosZip('./toto.zip')); // get infos of this ZIP archive (with files content)
$ARCHIVE->extractZip('./toto.zip', './1/'); */

class zip
{
    public function infosZip ($src, $data=true)
    {
        if (($zip = zip_open(realpath($src))))
        {
            while (($zip_entry = zip_read($zip)))
            {
                $path = zip_entry_name($zip_entry);
                if (zip_entry_open($zip, $zip_entry, "r"))
                {
                    $content[$path] = array (
                        'Ratio' => zip_entry_filesize($zip_entry) ? round(100-zip_entry_compressedsize($zip_entry) / zip_entry_filesize($zip_entry)*100, 1) : false,
                        'Size' => zip_entry_compressedsize($zip_entry),
                        'NormalSize' => zip_entry_filesize($zip_entry));
                    if ($data)
                        $content[$path]['Data'] = zip_entry_read($zip_entry, zip_entry_filesize($zip_entry));
                    zip_entry_close($zip_entry);
                }
                else
                    $content[$path] = false;
            }
            zip_close($zip);
            return $content;
        }
        return false;
    }
    public function extractZip ($src, $dest)
    {
        $zip = new ZipArchive;
        if ($zip->open($src)===true)
        {
            $zip->extractTo($dest);
            $zip->close();
            return true;
        }
        return false;
    }
    public function makeZip ($src, $dest)
    {
        $zip = new ZipArchive;
        $src = is_array($src) ? $src : array($src);
        if ($zip->open($dest, ZipArchive::CREATE) === true)
        {
            foreach ($src as $item)
                if (file_exists($item))
                    $this->addZipItem($zip, realpath(dirname($item)).'/', realpath($item).'/');
            $zip->close();
            return true;
        }
        return false;
    }
    private function addZipItem ($zip, $racine, $dir)
    {
        if (is_dir($dir))
        {
            $zip->addEmptyDir(str_replace($racine, '', $dir));
            $lst = scandir($dir);
                array_shift($lst);
                array_shift($lst);
            foreach ($lst as $item)
                $this->addZipItem($zip, $racine, $dir.$item.(is_dir($dir.$item)?'/':''));
        }
        elseif (is_file($dir))
            $zip->addFile($dir, str_replace($racine, '', $dir));
    }
}
?>